import os
import sys
import locale
import traceback
import gettext as gettext_module
from fsbc.Application import app


def initialize_locale(language=""):
    global translations

    print("initialize_locale language =", language)
    loc = language
    if not loc:
        try:
            loc, _charset = locale.getdefaultlocale()
            print("locale is", loc)
        except:
            print("exception while checking locale")
            loc = ""
        if not loc:
            loc = ""

    if not loc and sys.platform == "darwin":
        try:
            # noinspection PyUnresolvedReferences
            import CoreFoundation
            c_loc = CoreFoundation.CFLocaleCopyCurrent()
            loc = CoreFoundation.CFLocaleGetIdentifier(c_loc)
        except Exception:
            traceback.print_exc()
        print("mac locale", loc)

    dirs = [
        # os.path.join(os.getcwd(), "launcher", "share"),
        # os.path.join(os.getcwd(), "share"),
        # os.getcwd(),
        # "/usr/local/share",
        # "/usr/share"
        os.path.join(app.executable_dir(), "share"),
        os.path.join(app.executable_dir(), "..", "share"),
    ]
    if sys.platform == "darwin":
        dirs.insert(0, os.path.join(app.executable_dir(),
                                    "..", "Resources"))

    locale_base = None
    for dir in dirs:
        check = os.path.join(dir, "fs-uae-launcher", "share-dir")
        print("checking", check)
        if not os.path.exists(check):
            continue
        locale_base = os.path.join(dir, "locale")
        print("bindtextdomain fs-uae-launcher:", locale_base)
        gettext_module.bindtextdomain("fs-uae-launcher", locale_base)
        break
    
    mo_path = None
    if locale_base:
        print("find translations for", loc, "in local directory", locale_base)
        try:
            mo_path = gettext_module.find(
                "fs-uae-launcher", locale_base, [loc])
        except Exception as e:
            # a bug in openSUSE 12.2's gettext.py can cause an exception
            # in gettext.find (checking len of None).
            print(repr(e))
    else:
        print("no locale directory found")
    print("path to mo file:", mo_path)
    
    translations = gettext_module.translation(
        "fs-uae-launcher", locale_base, [loc], fallback=True)
    print("translations object:", translations)


def gettext(msg):
    return translations.gettext(msg)


def ngettext(n, msg1, msg2):
    return translations.ngettext(n, msg1, msg2)


translations = gettext_module.NullTranslations()
_ = gettext



from fsui.qt import Qt, QWidget


class DesktopWidget(QWidget):

    def __init__(self, parent):
        super().__init__(parent)
        self.setMinimumSize(640, 400)

        self.setAutoFillBackground(True)
        palette = self.palette()
        palette.setColor(self.backgroundRole(), Qt.gray)
        self.setPalette(palette)

        self.setFocusPolicy(Qt.NoFocus)
        self.setAttribute(Qt.WA_ShowWithoutActivating)



import os
from fsgs.mednafen.mednafen import MednafenRunner


# noinspection PyAttributeOutsideInit
class GameBoyAdvanceRunner(MednafenRunner):

    CONTROLLER = {
        "type": "gamepad",
        "description": "Built-in Controller",
        "mapping_name": "gameboyadvance",
    }

    PORTS = [
        {
            "description": "Controller",
            "types": [CONTROLLER]
        },
    ]

    def __init__(self, fsgs):
        super().__init__(fsgs)

    # def init_input(self):
    #     # self.inputs = [
    #     #     self.create_input(name="Controller",
    #     #             type="gameboyadvance",
    #     #             description="Built-in Gamepad"),
    #     # ]
    #     self.set_mednafen_input_order()

    def get_game_refresh_rate(self):
        # all GBA games should use a refresh rate of approx. 60.0 Hz
        # (or 30.0 Hz)
        return 59.73

    def mednafen_input_mapping(self, _):
        return {
            "A": "gba.input.builtin.gamepad.a",
            "B": "gba.input.builtin.gamepad.b",
            "L": "gba.input.builtin.gamepad.shoulder_l",
            "R": "gba.input.builtin.gamepad.shoulder_r",
            "UP": "gba.input.builtin.gamepad.up",
            "DOWN": "gba.input.builtin.gamepad.down",
            "LEFT": "gba.input.builtin.gamepad.left",
            "RIGHT": "gba.input.builtin.gamepad.right",
            "SELECT": "gba.input.builtin.gamepad.select",
            "START": "gba.input.builtin.gamepad.start",
        }

    def mednafen_post_configure(self):
        rom_file = self.get_game_file()
        cfg_file = os.path.splitext(rom_file)[0] + ".cfg"

        print("FIXME: temporarily removed support for custom SRAM type")
        # if os.path.exists(cfg_file):
        #     config = ConfigParser.ConfigParser()
        #     config.read(cfg_file)
        #     if config.has_option("game", "sram type"):
        #         type_file = os.path.join(
        #             self.context.game.state_dir,
        #             os.path.splitext(os.path.basename(rom_file))[0] + u".type")
        #         with open(type_file, "wb") as f:
        #             f.write(config.get("game", "sram type"))
        #             f.write("\n")

        print("FIXME: temporarily removed support for custom sav file")
        # sav_file = os.path.splitext(rom_file)[0] + u".sav"
        # if os.path.exists(sav_file):
        #     m = hashlib.md5()
        #     with open(rom_file, "rb") as f:
        #         while True:
        #             data = f.read(16384)
        #             if not data:
        #                 break
        #             m.update(data)
        #         md5sum = str(m.hexdigest())
        #     save_name = os.path.splitext(
        #         os.path.basename(rom_file))[0] + u"." + md5sum + u".sav"
        #     dest_path = os.path.join(self.get_state_dir(), save_name)
        #     if not os.path.exists(dest_path):
        #         shutil.copy(sav_file, dest_path)

        print("FIXME: temporarily removed supported for custom colormap")
        # self.colormap_temp = self.create_temp_file("color.map")
        # self.create_colormap(self.colormap_temp.path, 1.3)
        self.create_colormap(
            os.path.join(self.home.path, "gba.pal"), 1.3)
        # self.args.insert(0, self.colormap_temp.path)
        # self.args.insert(0, "-gba.colormap")

    def mednafen_rom_extensions(self):
        return [".gba"]

    def mednafen_scanlines_setting(self):
        return 33

    def mednafen_special_filter(self):
        return "nn2x"

    def mednafen_system_prefix(self):
        return "gba"

    # def mednafen_video_size(self):
    #     return 240, 160

    def create_colormap(self, path, gamma):
        with open(path, "wb") as f:
            for x in range(32768):
                r = (x & 0x1f) << 3
                g = ((x & 0x3e0) >> 5) << 3
                b = ((x & 0x7c00) >> 10) << 3

                r /= 255.0
                g /= 255.0
                b /= 255.0

                # h, l, s = colorsys.rgb_to_hls(r, g, b)
                # l = l ** gamma
                # r, g, b = colorsys.hls_to_rgb(h, l, s)

                r = r ** gamma
                g = g ** gamma
                b = b ** gamma

                f.write(bytes([int(r * 255)]))
                f.write(bytes([int(g * 255)]))
                f.write(bytes([int(b * 255)]))



import weakref
from fs_uae_launcher.Config import Config


class ConfigBehavior:

    def __init__(self, parent, names):
        parent.__amiga_enable_behavior = self
        self._parent = weakref.ref(parent)
        self._names = set(names)
        Config.add_listener(self)
        # FIXME
        # parent.destroyed.connect(self.on_destroy)

        # signal call for initial value
        for name in names:
            self.on_config(name, Config.get(name))

    def on_destroy(self, *args):
        Config.remove_listener(self)

    def on_config(self, key, value):
        if key in self._names:
            widget = self._parent()
            func = getattr(widget, "on_{0}_config".format(key))
            func(value)



from fsbc.util import unused
from fsgs.Database import Database
import fsui as fsui


class GameListView(fsui.ListView):

    def __init__(self, parent):
        fsui.ListView.__init__(self, parent)
        self.items = []

    def on_select_item(self, index):
        # index = self.get_index()
        database = Database.get_instance()
        info = database.get_game_info(self.items[index][0])
        self.set_game_info(info)

    def set_items(self, items):
        self.items = items
        self.set_item_count(len(self.items))

    def on_get_item_text(self, row, column):
        unused(column)
        return self.items[row][1]

    def on_get_item_tooltip(self, row, column):
        unused(column)
        return self.items[row][1]

    def search(self, search):
        print("search for", search)

        database = Database.get_instance()
        items = database.search_games(search)
        self.set_items(items)



import random
from fsgs.platform import PlatformHandler
from game_center.glui.font import Font
from game_center.glui.opengl import *
from game_center.glui.render import Render
from game_center.glui.state import State
from game_center.glui.texture import Texture


COORDS = 0.0, 1.0, 1.0, 0.0


def rectangle(x, y, w, h, z=0.0):
    # glBegin(GL_QUADS)
    glVertex3f(x, y, z)
    glVertex3f(x + w, y, z)
    glVertex3f(x + w, y + h, z)
    glVertex3f(x, y + h, z)
    # glEnd()


def texture_rectangle(x, y, w, h, z=0.0, s1=0.0, s2=1.0, t1=1.0, t2=0.0):
    glBegin(GL_QUADS)
    glTexCoord(s1, t1)
    glVertex3f(x, y, z)
    glTexCoord(s2, t1)
    glVertex3f(x + w, y, z)
    glTexCoord(s2, t2)
    glVertex3f(x + w, y + h, z)
    glTexCoord(s1, t2)
    glVertex3f(x, y + h, z)
    glEnd()


def static():
    s1 = random.uniform(0.0, 0.65)
    s2 = s1 + 0.35
    t1 = random.uniform(0.0, 0.65)
    t2 = t1 + 0.35
    # w, h = Texture.static.size
    # return Texture.static, (s1 * w, s2 * w, t1 * h, t2 * h)
    return Texture.static, (s1, s2, t1, t2)


last_item_id = None
item_time = 0


def render_bottom_bar_transparent():
    Render.hd_perspective()
    fs_emu_blending(True)
    Texture.bottom_bar.render(0, 0, 1920, 380, 0.3, opacity=1.0)
    item = State.current_menu.selected_item
    render_bottom_bar_text(item)


def render_bottom_bar():
    Render.hd_perspective()
    item = State.current_menu.selected_item
    render_bottom_bar_screens(item)


def render_bottom_bar_screens(item):
    global last_item_id
    global item_time
    if id(item) != last_item_id:
        last_item_id = id(item)
        item_time = State.time

    glColor3f(1.0, 1.0, 1.0)
    fs_emu_texturing(True)
    fs_emu_blending(False)

    for i in range(3, -1, -1):
        if State.time - item_time > 0.1 + 0.05 * i:
            texture = item.get_screen_texture(2 + i)
        else:
            texture = None
        if texture:
            # coords = (0, texture.w, texture.h, 0)
            coords = COORDS
        else:
            texture, coords = static()
            # while rendering static, we must continue rendering
            Render.dirty = True

        # print(repr(texture))
        # w, h = texture.size
        # glBindTexture(GL_TEXTURE_2D, texture)
        texture.bind()
        # coords = (coords[0] * w, coords[1] * w, coords[2] * h, coords[3] * h)
        texture_rectangle(4 + 539 + 345 * i, 34, 308, 193, 0.2, *coords)
    if State.time - item_time > 0.1:
        texture = item.get_screen_texture(1)
    else:
        texture = None
    if texture:
        coords = COORDS
        # coords = (0, texture.w, texture.h, 0)
    else:
        texture, coords = static()
        # while rendering static, we must continue rendering
        Render.dirty = True
    # w, h = texture.size
    # glBindTexture(GL_TEXTURE_2D, texture)
    texture.bind()
    # coords = (coords[0] * w, coords[1] * w, coords[2] * h, coords[3] * h)
    
    texture_rectangle(34, 34, 472, 295, 0.2, *coords)

    # fs_emu_blending(True)
    # Texture.info_bar.render(0, 0, 1920, 380, opacity=1.0)
    #
    # Texture.screen_border_1.render(0, 0, 540, 363, 0.1)
    # for i in range(4):
    #     Texture.screen_border_2.render(509 + 345 * i, 0, 376, 261,
    #             0.11 + 0.01 * i)
    #
    # glColor4f(0.7, 0.7, 0.7, 0.7)
    # #glBindTexture(GL_TEXTURE_2D, Texture.gloss)
    # Texture.gloss.bind()
    # for i in range(4):
    #     texture_rectangle(4 + 539 + 345 * i, 34, 308, 193, 0.3)
    # texture_rectangle(34, 34, 472, 295, 0.3)
    #
    # #if item_index == menu.get_selected_index():
    # #    State.preview_screen.set_screen(item.get_screen_texture(1))


def render_bottom_bar_text(item):
    # x = State.gl_left + 0.05
    strength = 0.9
    x = 544
    y = 290

    title = item.title.upper()
    # subtitle = State.current_menu.selected_item.subtitle.upper()
    # supertitle = State.current_menu.selected_item.supertitle.upper()
    if title:
        Render.text(title, Font.title_font, x, y, 1920 - x - 170,
                    shadow=True, color=(1.0, 1.0, 1.0, 1.0 * strength))

    year_text = str(getattr(State.current_menu.selected_item,
                            "year", "") or "").upper()
    if year_text:
        tw, th = Render.measure_text(year_text, Font.title_font)
        Render.text(year_text, Font.title_font, 1920 - 30 - tw, y, 0,
                    shadow=True, color=(1.0, 1.0, 1.0, 1.0 * strength))

    # strength = 0.4
    color = (0x6e / 0xff, 0x8b / 0xff, 0x96 / 0xff, 1.0)
    # y = 253
    y = 258
    text_str = ""
    companies = set()
    publisher_text = (getattr(State.current_menu.selected_item,
                              "publisher", "") or "").upper()
    for text in publisher_text.split("/"):
        text = text.strip()
        if text:
            if text not in companies:
                text_str = text_str + u" \u00b7 " + text
                companies.add(text)
    developer_text = (getattr(State.current_menu.selected_item,
                              "developer", "") or "").upper()
    for text in developer_text.split("/"):
        text = text.strip()
        if text:
            if text not in companies:
                text_str = text_str + u" \u00b7 " + text
                companies.add(text)
    if len(text_str) > 3:
        text_str = text_str[3:]  # remove initial middle dot
        Render.text(text_str, Font.subtitle_font, x, y, 0, shadow=True,
                    color=color)

    platform_str = str(getattr(State.current_menu.selected_item, "platform", "") or "")
    if platform_str:
        platform_str = PlatformHandler.get_platform_name(platform_str).upper()
    if len(platform_str) >= 3:
        tw, th = Render.measure_text(platform_str, Font.subtitle_font)
        Render.text(platform_str, Font.subtitle_font, 1920 - 30 - tw, y, 0, shadow=True,
                    color=color)
    return

    # Render.ortho_perspective()
    # #render_header(color=(0.5, 0.5, 0.5), text="Ku Game System",
    # #         offset=header_offset)
    #
    # fs_emu_blending(True)
    # glBindTexture(GL_TEXTURE_2D, Texture.bottom_bar)
    # glColor4f(0.85, 0.85, 0.85, 0.85)
    # glBegin(GL_QUADS)
    # glTexCoord2f(0.0, 1.0)
    # glVertex2f(State.gl_left, -1.0)
    # glTexCoord2f(1.0, 1.0)
    # glVertex2f(State.gl_right, -1.0)
    # glTexCoord2f(1.0, 0.0)
    # glVertex2f(State.gl_right, -0.75)
    # glTexCoord2f(0.0, 0.0)
    # glVertex2f(State.gl_left, -0.75)
    # glEnd()
    #
    # top_mode = State.top_menu == State.navigatable
    # config_mode = bool(State.config_menu)
    #
    # if top_mode:
    #     strength = 0.4
    # else:
    #     strength = 1.0
    #
    #
    # x = State.gl_left + 0.05
    # y = -0.89
    #
    # title = State.current_menu.selected_item.title.upper()
    # subtitle = State.current_menu.selected_item.subtitle.upper()
    # supertitle = State.current_menu.selected_item.supertitle.upper()
    #
    # if supertitle:
    #     tw, th = Render.text(supertitle, Font.title_font,
    #             x, y, 0, shadow=True,
    #             color=(1.0, 1.0, 1.0, 1.0 * strength))
    #     x += tw + 0.03
    # if title:
    #     tw, th = Render.text(title, Font.title_font,
    #             x, y, 0, shadow=True,
    #             color=(1.0, 1.0, 1.0, 1.0 * strength))
    #     x += tw + 0.03
    # if subtitle:
    #     tw, th = Render.text(subtitle, Font.title_font,
    #             x, y, 0, shadow=True,
    #             color=(1.0, 1.0, 1.0, 1.0 * strength))
    #     x += tw + 0.03
    #
    # x = State.gl_left + 0.05
    # y = -0.96
    #
    #
    # options = {"dot": False,
    #     "x": State.gl_left + 0.05,
    #     "y": -0.96,
    #     "texts": set()
    # }
    # def text(t, measure=None):
    #     if not t:
    #         return
    #     if t in options["texts"]:
    #         return
    #     if options["dot"]:
    #         tw, th = Render.text(u"\u00b7", Font.subtitle_font,
    #                 options["x"], options["y"], 0, shadow=True,
    #                 color=(1.0, 1.0, 1.0, 1.0 * strength * 0.25))
    #         options["x"] += tw + 0.02
    #     tw, th = Render.text(t, Font.subtitle_font,
    #             options["x"], options["y"], 0, shadow=True,
    #             color=(1.0, 1.0, 1.0, 1.0 * strength * 0.33))
    #     if measure:
    #         tw, th = Render.measure_text(measure, Font.subtitle_font)
    #     options["x"] += tw + 0.02
    #     options["dot"] = True
    #     options["texts"].add(t)
    #
    # platform_text = str(getattr(State.current_menu.selected_item, "platform", "") or "").upper()
    # text(platform_text)
    #
    # year_text = str(getattr(State.current_menu.selected_item, "year", "") or "").upper()
    # text(year_text, measure="0000")
    #
    # publisher_text = (getattr(State.current_menu.selected_item, "publisher", "") or "").upper()
    # for publisher_text in publisher_text.split("/"):
    #     text(publisher_text.strip())



from .mess import MESSRunner


class Atari5200Runner(MESSRunner):

    CONTROLLER = {
        "type": "controller",
        "description": "Controller",
        "mapping_name": "atari5200",
    }

    PORTS = [
        {
            "description": "1st Controller",
            "types": [CONTROLLER]
        }, {
            "description": "2nd Controller",
            "types": [CONTROLLER]
        },
    ]

    def get_game_refresh_rate(self):
        # no PAL version of Atari 5200
        return 59.94

    def mess_configure(self):
        self.mess_configure_cartridge()

    def mess_input_mapping(self, port):
        return {
            "START": 'type="P#_START"',
            "PAUSE": 'tag="keypad_2" type="KEYPAD" mask="1" defvalue="0"',
            "RESET": 'tag="keypad_1" type="KEYPAD" mask="1" defvalue="0"',
            "RIGHT": ('type="P#_AD_STICK_X"', 'increment'),
            "LEFT": ('type="P#_AD_STICK_X"', 'decrement'),
            "UP": ('type="P#_AD_STICK_Y"', 'decrement'),
            "DOWN": ('type="P#_AD_STICK_Y"', 'increment'),
            "1": "P#_BUTTON1",
            "2": "P#_BUTTON2",
            "PAD#": 'tag="keypad_0" type="KEYPAD" mask="2" defvalue="0"',
            "PAD0": 'tag="keypad_0" type="KEYPAD" mask="4" defvalue="0"',
            "PAD*": 'tag="keypad_0" type="KEYPAD" mask="8" defvalue="0"',
            "PAD9": 'tag="keypad_1" type="KEYPAD" mask="2" defvalue="0"',
            "PAD8": 'tag="keypad_1" type="KEYPAD" mask="4" defvalue="0"',
            "PAD7": 'tag="keypad_1" type="KEYPAD" mask="8" defvalue="0"',
            "PAD6": 'tag="keypad_2" type="KEYPAD" mask="2" defvalue="0"',
            "PAD5": 'tag="keypad_2" type="KEYPAD" mask="4" defvalue="0"',
            "PAD4": 'tag="keypad_2" type="KEYPAD" mask="8" defvalue="0"',
            "PAD3": 'tag="keypad_3" type="KEYPAD" mask="2" defvalue="0"',
            "PAD2": 'tag="keypad_3" type="KEYPAD" mask="4" defvalue="0"',
            "PAD1": 'tag="keypad_3" type="KEYPAD" mask="8" defvalue="0"',
        }

    def mess_romset(self):
        return "a5200", A5200_ROMS


A5200_ROMS = {
    "6ad7a1e8c9fad486fbec9498cb48bf5bc3adc530": "5200.rom",
    "1d2a3f00109d75d2d79fecb565775eb95b7d04d5": "5200a.rom",
}



import weakref
from game_center.glui.bezier import Bezier
from game_center.glui.render import State


class AnimationSystem(object):

    animation_list = []

    @classmethod
    def is_active(cls):
        # for refobj in cls.animation_list:
        #     obj = refobj()
        #     if obj == None:
        #         deleted.append(refobj)
        # print "is_active"
        # print cls.animation_list
        return len(cls.animation_list) > 0

    @classmethod
    def update(cls, t=None):
        t = t or State.time
        deleted = []
        for refobj in cls.animation_list:
            obj = refobj()
            if obj is None:
                deleted.append(refobj)
            else:
                if not obj.update(t):
                    deleted.append(refobj)
        for d in deleted:
            cls.animation_list.remove(d)


class AnimateValueLinear(object):

    def __init__(
            self, what, from_value, from_time, to_value, to_time, run=True):
        self.what = what
        self.from_value = from_value
        self.to_value = to_value
        self.from_time = from_time
        self.to_time = to_time
        if run:
            AnimationSystem.animation_list.append(weakref.ref(self))

    @property
    def max_time(self):
        return self.to_time

    def update(self, t=None):
        t = t or State.time
        # print self.from_value, self.to_value
        progress = (t - self.from_time) / (
            self.to_time - self.from_time)
        progress = max(0.0, min(1.0, progress))
        setattr(self.what[0], self.what[1],
                self.from_value + progress * (self.to_value - self.from_value))
        return t < self.to_time


class AnimateValueBezier(object):

    def __init__(
            self, what, x0, t0, x1, t1, x2, t2, x3, t3, points=50, run=True):
        self.what = what
        # self.from_value = from_value
        # self.to_value = to_value
        # self.from_time = from_time
        # self.to_time = to_time
        self.params = (x0, t0, x1, t1, x2, t2, x3, t3)
        self.curve = Bezier.bezier((t0, x0), (t1, x1), (t2, x2), (t3, x3))
        if run:
            AnimationSystem.animation_list.append(weakref.ref(self))

    @property
    def max_time(self):
        return self.curve[-1][0]

    def update(self, t=None):
        # print "AnimateValue.update"
        # print self.from_value, self.to_value

        # progress = pygame.get_ticks() - from_time
        # totaltime = to_time - from_time

        x0, t0, x1, t1, x2, t2, x3, t3 = self.params
        # t = t or max(t0, min(t3, pygame.time.get_ticks()))
        t = t or State.time
        # t = max(t0, min(t3, t))
        if t < t0:
            t = t0
        elif t > t3:
            t = t3
        # print t0, t, t3
        # print type(t0), type(t), type(t3)

        # progress = (pygame.time.get_ticks() - self.from_time) / (
        #         self.to_time - self.from_time)
        # progress = max(0.0, min(1.0, progress))
        # pygame.time.get_ticks() - t0

        x = Bezier.interpolate(self.curve, t)
        # print x
        setattr(self.what[0], self.what[1], x)
        # print t, t3, t < t3
        return t < t3


class AnimationSequence(object):

    def __init__(self, anim_seq):
        self.anim_seq = anim_seq
        AnimationSystem.animation_list.append(weakref.ref(self))

    def update(self, t=None):
        t = t or State.time
        for anim in self.anim_seq:
            if t > anim.max_time:
                continue
            anim.update(t)
            break
        else:
            return self.anim_seq[-1].update()
        return True



import os
import io
import traceback
from fsbc.zipfile import ZipFile

archive_extensions = [".zip", ".rp9"]

try:
    from lhafile import LhaFile
except ImportError:
    traceback.print_exc()
    print("LhaFile module import problem")
    LhaFile = None
else:
    archive_extensions.append(".lha")

try:
    from fsbc.SevenZipFile import SevenZipFile
except ImportError:
    traceback.print_exc()
    print("SevenZipFile module import problem")
    SevenZipFile = None
else:
    archive_extensions.append(".7z")


class ZipHandler(object):

    def __init__(self, path):
        self.path = path
        self.zip = ZipFile(self.path, "r")

    def list_files(self, sub_path):
        if sub_path:
            return
        return self.zip.namelist()
        # for name in self.zip.namelist():
        #     #if name.endswith(str("/")):
        #     #if name.endswith("/"):
        #     #    continue
        #     #yield self.decode_name(name)
        #     yield name

    def open(self, name):
        return self.zip.open(name)

    def exists(self, name):
        try:
            self.zip.getinfo(name)
        except KeyError:
            return False
        else:
            return True

    # def encode_name(self, name):
    #     name = name.replace("\\", "/")
    #     name = name.replace("%5f", "\\")
    #     name = name.replace("%25", "%")
    #     #name = name.encode("CP437")
    #     name = name.encode("ISO-8859-1")
    #     return name
    #
    # def decode_name(self, name):
    #     #name = name.decode("CP437")
    #     name = name.decode("ISO-8859-1")
    #     name = name.replace("%", "%25")
    #     name = name.replace("\\", "%5f")
    #     name = name.replace("/", os.sep)
    #     return name


class SevenZipHandler(object):

    def __init__(self, path):
        self.path = path
        self.zip = SevenZipFile(self.path, "r")

    def list_files(self, sub_path):
        if sub_path:
            return
        return self.zip.namelist()

    def read(self, name):
        return self.zip.read(name)

    def open(self, name):
        data = self.zip.read(name)
        return io.BytesIO(data)

    def exists(self, name):
        try:
            self.zip.getinfo(name)
        except KeyError:
            return False
        else:
            return True


class LhaHandler(object):

    def __init__(self, path):
        self.path = path
        self.zip = LhaFile(self.path, "r")

    def list_files(self, sub_path):
        if sub_path:
            return
        for name in self.zip.namelist():
            # if name.endswith(str("/")):
            #     continue
            yield self.decode_name(name)

    def open(self, name):
        name = self.encode_name(name)
        # LhaFile does not have open method
        data = self.zip.read(name)
        return io.BytesIO(data)

    def exists(self, name):
        name = self.encode_name(name)
        items = self.zip.infolist()
        for item in items:
            if item.filename == name:
                return True
        return False

        # try:
        #     self.zip.getinfo(name)
        # except KeyError:
        #     return False
        # else:
        #     return True

    def encode_name(self, name):
        name = name.replace("\\", "/")
        name = name.replace("%5f", "\\")
        name = name.replace("%25", "%")

        # FIXME: a little hack here, LhaFile uses os.sep
        # as path separator
        name = name.replace("/", os.sep)

        # name = name.encode("ISO-8859-1")
        return name

    def decode_name(self, name):
        # print("decode_name", name)

        # name = name.decode("ISO-8859-1")
        # FIXME: a little hack here, LhaFile uses os.sep
        # as path separator, normalizing to /
        name = name.replace(os.sep, "/")

        name = name.replace("%", "%25")
        name = name.replace("\\", "%5f")
        name = name.replace("/", os.sep)
        return name


class NullHandler(object):

    def __init__(self, path):
        self.path = path

    def list_files(self, _):
        return []

    def open(self, path):
        return open(path, "rb")


class Archive(object):

    extensions = archive_extensions

    def __init__(self, path):
        self.path, self.sub_path = self.split_path(path)
        self._handler = None

    def join(self, base, *args):
        return os.path.join(base, *args)

    def dirname(self, path):
        return os.path.dirname(path)

    def split_path(self, path):
        print("split_path", path)
        if "#/" in path:
            parts = path.rsplit("#/", 1)
            archive = parts[0]
            # if not archive[-1] in "/\\" and os.path.exists(archive):
            if os.path.isfile(archive):
                sub_path = parts[1] if len(parts) > 1 else ""
                return archive, sub_path

        parts = path.replace("\\", "/").split("/")
        for i, part in enumerate(parts):
            n, ext = os.path.splitext(part)
            ext = ext.lower()
            if ext in archive_extensions:
                # FIXME: should also check that it isn't a dir
                path = str(os.sep).join(parts[:i + 1])
                sub_path = str(os.sep).join(parts[i + 1:])
                return path, sub_path
        return path, ""

    def get_handler(self):
        if self._handler is not None:
            return self._handler
        print("get_handler", self.path)
        name, ext = os.path.splitext(self.path)
        ext = ext.lower()
        if ext == ".7z" and SevenZipFile is not None:
            self._handler = SevenZipHandler(self.path)
            return self._handler

        try:
            self._handler = ZipHandler(self.path)
        except Exception as e:
            if ext == ".zip":
                print(repr(e))
            try:
                self._handler = LhaHandler(self.path)
            except Exception as e:
                if ext == ".lha":
                    print(repr(e))
                self._handler = NullHandler(self.path)
        return self._handler

    def list_files(self):
        result = []
        print(self.get_handler())
        for item in self.get_handler().list_files(self.sub_path):
            # result.append(os.path.join(self.path, item))
            result.append(self.path + "#/" + item)
        return result

    def exists(self, path):
        path, sub_path = self.split_path(path)
        # print(path, self.path)
        assert path == self.path
        if not sub_path:
            # print("os.path.exists", path)
            return os.path.exists(path)
        return self.get_handler().exists(sub_path)

    def open(self, path):
        # print("open", repr(path))
        path, sub_path = self.split_path(path)
        # print(path, self.path)
        assert path == self.path
        if not sub_path:
            return open(path, "rb")
        return self.get_handler().open(sub_path)

    def copy(self, path, dest):
        ifo = self.open(path)
        with open(dest, "wb") as ofo:
            while True:
                data = ifo.read(65536)
                if not data:
                    break
                ofo.write(data)



import os
import subprocess
from .FSUAE import FSUAE

try:
    getcwd = os.getcwdu
except AttributeError:
    getcwd = os.getcwd


class FSUAEDeviceHelper(object):

    @classmethod
    def start_with_args(cls, args, **kwargs):
        print("FSUAE.start_with_args:", args)
        exe = cls.find_executable()
        print("current dir (cwd): ", getcwd())
        print("using fs-uae executable:", exe)
        args = [exe] + args
        print(args)
        env = os.environ.copy()
        FSUAE.add_environment_from_settings(env)
        process = subprocess.Popen(
            args, env=env, stdin=subprocess.PIPE,
            stderr=subprocess.PIPE, **kwargs)
        return process

    @classmethod
    def find_executable(cls):
        return FSUAE.find_executable("fs-uae-device-helper")



from fsgs.mednafen.mednafen import MednafenRunner


class NintendoRunner(MednafenRunner):

    CONTROLLER = {
        "type": "gamepad",
        "description": "Gamepad",
        "mapping_name": "nintendo",
    }

    PORTS = [
        {
            "description": "1st Controller",
            "types": [CONTROLLER]
        }, {
            "description": "2nd Controller",
            "types": [CONTROLLER]
        },
    ]

    def __init__(self, fsgs):
        super().__init__(fsgs)

    def force_aspect_ratio(self):
        return 4.0 / 3.0

    def mednafen_extra_graphics_options(self):
        options = []
        if self.is_pal():
            options.extend(["-nes.pal", "1"])
        else:
            options.extend(["-nes.pal", "0"])
        if self.nes_clip_sides():
            options.extend(["-nes.clipsides", "1"])
        return options

    def mednafen_input_mapping(self, port):
        if port == 0:
            return {
                "A": "nes.input.port1.gamepad.a",
                "B": "nes.input.port1.gamepad.b",
                "UP": "nes.input.port1.gamepad.up",
                "DOWN": "nes.input.port1.gamepad.down",
                "LEFT": "nes.input.port1.gamepad.left",
                "RIGHT": "nes.input.port1.gamepad.right",
                "SELECT": "nes.input.port1.gamepad.select",
                "START": "nes.input.port1.gamepad.start",
            }
        elif port == 1:
            return {
                "A": "nes.input.port2.gamepad.a",
                "B": "nes.input.port2.gamepad.b",
                "UP": "nes.input.port2.gamepad.up",
                "DOWN": "nes.input.port2.gamepad.down",
                "LEFT": "nes.input.port2.gamepad.left",
                "RIGHT": "nes.input.port2.gamepad.right",
                "SELECT": "nes.input.port2.gamepad.select",
                "START": "nes.input.port2.gamepad.start",
            }

    def mednafen_rom_extensions(self):
        return [".nes"]

    def mednafen_system_prefix(self):
        return "nes"

    def mednafen_video_size(self):
        # FIXME
        if self.is_pal():
            size = (256, 240)
        else:
            size = (256, 224)
        if self.nes_clip_sides():
            size = (size[0] - 16, size[1])
        return size

    def nes_clip_sides(self):
        # FIXME: Sane default? -Or enable this in a per-game config
        # instead? SMB3 looks better with this
        return True



import fsui as fsui
from ...I18N import gettext
from .ScanPathsGroup import ScanPathsGroup
from fsui.extra.iconheader import NewIconHeader


class ScanSettingsPage(fsui.Panel):

    def __init__(self, parent):
        fsui.Panel.__init__(self, parent)
        self.layout = fsui.VerticalLayout()
        # self.layout.set_padding(20, 20, 20, 20)

        self.icon_header = NewIconHeader(
            self, fsui.Icon("indexing-settings", "pkg:fs_uae_workspace"),
            gettext("File Database Settings"),
            gettext("Choose what folders you want to scan for Amiga "
                    "files"))
        self.layout.add(self.icon_header, fill=True, margin_bottom=20)

        # label = fsui.Label(self, gettext("List of Folders to Index:"))
        # self.layout.add(label, margin=10)

        self.layout.add(fsui.MultiLineLabel(
            self, gettext("Choose what folders you want to scan for Amiga "
                          "files")), fill=True, margin_bottom=10)

        self.scan_paths_group = ScanPathsGroup(self)
        self.layout.add(self.scan_paths_group, fill=True, expand=True)

        # label = fsui.HeadingLabel(self, gettext("Additional Options"))
        # self.layout.add(label, margin=10, margin_bottom=20)
        #
        # def add_option(name):
        #     self.layout.add(
        #         OptionUI.create_group(self, name), fill=True, margin=10)
        #
        # add_option("builtin_configs")
        # add_option("kickstart_setup")



from fsbc.util import unused
import fsui as fsui
from fsgs.amiga.Amiga import Amiga
from ..Config import Config
from .FloppiesGroup import FloppiesGroup


class RemovableMediaGroup(FloppiesGroup):

    def __init__(self, parent, drives):
        FloppiesGroup.__init__(self, parent, drives)
        self.layout3 = fsui.HorizontalLayout()
        self.layout.add(self.layout3, fill=True)

        self.layout3.add_spacer(0, expand=True)

        self.cd_mode = False

        self.update_media_type()
        Config.add_listener(self)

    def on_destroy(self):
        Config.remove_listener(self)

    def on_config(self, key, value):
        unused(value)
        if key == "amiga_model":
            self.update_media_type()

    def update_media_type(self):
        self.set_cd_mode(Amiga.is_cd_based(Config))

    def set_cd_mode(self, cd_mode):
        if self.cd_mode == cd_mode:
            return
        self.cd_mode = cd_mode
        for selector in self.selectors:
            selector.set_cd_mode(cd_mode)
        self.update_heading_label()
        self.selectors[1].enable(not self.cd_mode)



from operator import itemgetter
from fs_uae_launcher.Config import Config
from fs_uae_launcher.DeviceManager import DeviceManager
from fs_uae_launcher.I18N import gettext
from fs_uae_launcher.UpdateManager import UpdateManager
from fs_uae_launcher.ui.SetupDialog import SetupDialog
from fs_uae_launcher.ui.behaviors.configbehavior import ConfigBehavior
from fs_uae_launcher.ui.behaviors.settingsbehavior import SettingsBehavior
from fs_uae_launcher.ui.download import DownloadGameWindow
from fs_uae_launcher.ui.statusbar.StatusElement import StatusElement
from fsgs.amiga.Amiga import Amiga
from fsgs.context import fsgs
from fsui import Image
import fsui

ERROR_LEVEL = 0
WARNING_LEVEL = 1
NOTICE_LEVEL = 2
JOYSTICK_KEYS = ["joystick_port_0", "joystick_port_1", "joystick_port_2",
                 "joystick_port_3", "joystick_port_0_mode",
                 "joystick_port_1_mode", "joystick_port_2_mode",
                 "joystick_port_3_mode"]


class WarningsElement(StatusElement):

    def __init__(self, parent):
        StatusElement.__init__(self, parent)
        self.error_icon = Image("fs_uae_launcher:res/16/error.png")
        self.warning_icon = Image("fs_uae_launcher:res/16/warning_3.png")
        self.notice_icon = Image("fs_uae_launcher:res/16/information.png")
        self.icons = [
            self.error_icon,
            self.warning_icon,
            self.notice_icon,
        ]
        self.coordinates = []
        self.warnings = []

        self.game_notice = ""
        self.variant_notice = ""
        self.variant_warning = ""
        self.variant_error = ""

        self.joy_emu_conflict = ""
        self.using_joy_emu = False

        self.kickstart_file = ""
        self.x_kickstart_file_sha1 = ""

        self.update_available = ""
        self.__error = ""

        self.x_missing_files = ""
        self.download_page = ""
        self.download_file = ""

        self.amiga_model = ""
        self.amiga_model_calculated = ""
        self.chip_memory = ""
        self.chip_memory_calculated = 0

        ConfigBehavior(self, [
            "x_game_notice", "x_variant_notice", "x_variant_warning",
            "x_variant_error", "x_joy_emu_conflict", "amiga_model",
            "x_kickstart_file_sha1", "kickstart_file", "download_page",
            "download_file", "x_missing_files", "__error",
            "chip_memory"])
        SettingsBehavior(self, ["__update_available"])

        self.on_config("joystick_port_0", Config.get("joystick_port_0"))
        Config.add_listener(self)

    def on_destroy(self):
        Config.remove_listener(self)

    def on_amiga_model_config(self, value):
        Config.update_kickstart()

        if value != self.amiga_model:
            self.amiga_model = value
            self.amiga_model_calculated = value.split("/")[0]
            self.rebuild_warnings_and_refresh()

    def on_chip_memory_config(self, value):
        if value != self.chip_memory:
            self.chip_memory = value
            try:
                self.chip_memory_calculated = int(value or "0")
            except Exception:
                self.chip_memory_calculated = -1
            self.rebuild_warnings_and_refresh()

    def on___error_config(self, value):
        if value != self.__error:
            self.__error = value
            self.rebuild_warnings_and_refresh()

    def on_x_missing_files_config(self, value):
        if value != self.x_missing_files:
            self.x_missing_files = value
            self.rebuild_warnings_and_refresh()

    def on_download_page_config(self, value):
        if value != self.download_page:
            self.download_page = value
            self.rebuild_warnings_and_refresh()

    def on_download_file_config(self, value):
        if value != self.download_file:
            self.download_file = value
            self.rebuild_warnings_and_refresh()

    def on_kickstart_file_config(self, value):
        if value != self.kickstart_file:
            self.kickstart_file = value
            self.rebuild_warnings_and_refresh()

    def on_x_kickstart_file_sha1_config(self, value):
        if value != self.x_kickstart_file_sha1:
            self.x_kickstart_file_sha1 = value
            self.rebuild_warnings_and_refresh()

    def on_x_game_notice_config(self, value):
        if value != self.game_notice:
            self.game_notice = value
            self.rebuild_warnings_and_refresh()

    def on_x_variant_notice_config(self, value):
        if value != self.variant_notice:
            self.variant_notice = value
            self.rebuild_warnings_and_refresh()

    def on_x_variant_warning_config(self, value):
        if value != self.variant_warning:
            self.variant_warning = value
            self.rebuild_warnings_and_refresh()

    def on_x_variant_error_config(self, value):
        if value != self.variant_error:
            self.variant_error = value
            self.rebuild_warnings_and_refresh()

    def on_x_joy_emu_conflict_config(self, value):
        print("\n\n\nGOT x_joy_emu_conflict\n\n\n")
        if value != self.joy_emu_conflict:
            self.joy_emu_conflict = value
            self.rebuild_warnings_and_refresh()

    def on_config(self, key, _):
        if key in JOYSTICK_KEYS:
            prev_value = self.using_joy_emu
            devices = DeviceManager.get_devices_for_ports(Config)
            for device in devices:
                if device.id == "keyboard":
                    self.using_joy_emu = True
                    break
            else:
                self.using_joy_emu = False
            if prev_value != self.using_joy_emu:
                self.rebuild_warnings_and_refresh()

    def on___update_available_setting(self, value):
        if value != self.update_available:
            self.update_available = value
            self.rebuild_warnings_and_refresh()

    def paint_element(self, dc):
        self.coordinates.clear()
        dc.set_font(dc.get_font())
        x = 6
        w, h = self.size
        for level, warning, handler in self.warnings:
            icon = self.icons[level]
            start = x
            dc.draw_image(icon, x, 6)
            x += 16 + 6
            tw, th = dc.measure_text(warning)
            dc.draw_text(warning, x, (h - th) / 2)
            x += tw + 6 + 16
            stop = x
            self.coordinates.append((start, stop, handler))

    def on_left_down(self):
        from fsui.qt import QCursor
        # noinspection PyArgumentList
        p = self.mapFromGlobal(QCursor.pos())
        for start, stop, handler in self.coordinates:
            if start <= p.x() < stop and handler:
                # print(start, stop, handler)
                getattr(self, handler)()

    def on_update(self):
        UpdateManager.start_update(self.update_available)

    def on_kickstart_warning(self):
        text = ("The Kickstart ROM for the chosen Amiga model was not found "
                "on your system.\n\n"
                "A replacement Kickstart ROM from the AROS project is used "
                "instead. Compatibility will be lower than if you use an "
                "original Kickstart ROM.\n\n"
                "You can use the file database scan function or the import "
                "wizards if you own the Kickstart ROM.\n\n"
                "Original Kickstart ROMs can be purchased online as part "
                "of Cloanto's Amiga Forever package, or you can extract "
                "a Kickstart from a real Amiga.\n\n"
                "If you want to use the replacement Kickstart, you can "
                "ignore this warning, or explicitly change the Kickstart "
                "to \"Internal\" to dismiss this warning.")
        fsui.show_warning(text, gettext("Using Kickstart ROM Replacement"))

    def on_import_kickstarts(self):
        SetupDialog(self.get_window()).show()

    def on_download_page(self):
        DownloadGameWindow(self.get_window(), fsgs).show()

    def rebuild_warnings(self):
        self.warnings = []

        if self.using_joy_emu and self.joy_emu_conflict:
            self.warnings.append((WARNING_LEVEL, self.joy_emu_conflict, ""))

        for name in ["variant_notice", "game_notice"]:
            value = getattr(self, name)
            if not value:
                continue
            if value.startswith("WARNING: "):
                level = WARNING_LEVEL
                message = value[9:]
            else:
                level = NOTICE_LEVEL
                message = value
            self.warnings.append((level, message, ""))
        if self.variant_warning:
            self.warnings.append((WARNING_LEVEL, self.variant_warning, ""))
        if self.variant_error:
            self.warnings.append((ERROR_LEVEL, self.variant_error, ""))

        if self.update_available:
            text = gettext("Update Available: {version}").format(
                version=self.update_available)
            self.warnings.append((NOTICE_LEVEL, text, "on_update"))

        if self.x_kickstart_file_sha1 == Amiga.INTERNAL_ROM_SHA1 and \
                self.kickstart_file != "internal":
            # text = gettext("Compatibility Issue")
            # self.warnings.append((ERROR_LEVEL, text, "on_kickstart_warning"))
            text = gettext("Using Kickstart ROM Replacement")
            self.warnings.append((WARNING_LEVEL, text, "on_kickstart_warning"))
            text = gettext("Click to Import Kickstart ROMs")
            self.warnings.append((NOTICE_LEVEL, text, "on_import_kickstarts"))

        if is_warning(self.x_missing_files):
            if self.download_file:
                text = gettext("Automatic Download")
                self.warnings.append((NOTICE_LEVEL, text, ""))
            elif self.download_page:
                text = gettext("Click to Download Game")
                self.warnings.append((WARNING_LEVEL, text, "on_download_page"))
            else:
                text = gettext("Missing Game Files")
                self.warnings.append((ERROR_LEVEL, text, ""))

        if self.__error:
            self.warnings.append((ERROR_LEVEL, self.__error, ""))

        if self.chip_memory_calculated and \
                self.chip_memory_calculated < 2048 and \
                self.amiga_model_calculated in ["A1200", "A4000"]:
            text = gettext("{amiga_model} with < 2 MB chip memory"
                           "").format(amiga_model=self.amiga_model)
            self.warnings.append((WARNING_LEVEL, text, ""))

        self.warnings.sort(key=itemgetter(0))

    def rebuild_warnings_and_refresh(self):
        self.rebuild_warnings()
        self.refresh()


def is_warning(w):
    if w is None:
        return False
    if isinstance(w, str):
        return bool(w)
    if isinstance(w, bool):
        return w
    return w[0] or w[1]



import fsbc.system
import nose.tools


def test_mypy():
    # from nose.plugins.skip import SkipTest
    # raise SkipTest()
    import fstd.mypy
    fstd.mypy.check_module(fsbc.system.__name__)


def test_doctest():
    import doctest
    failure_count, test_count = doctest.testmod(fsbc.system)
    nose.tools.assert_equals(failure_count, 0)



import fsui as fsui
from fsgs.amiga.Amiga import Amiga
from ...Config import Config


class MemoryWidget(fsui.Group):

    def __init__(self, parent, label, option_key, values):
        fsui.Group.__init__(self, parent)
        self.option_key = option_key
        self.values = values
        self.layout = fsui.HorizontalLayout()

        # label = fsui.Label(self, label + ":")
        # self.layout.add(label, expand=True, margin_right=10)
        self.checkbox = fsui.CheckBox(self, label + ":")
        self.checkbox.on_change = self.on_checkbox_change
        self.layout.add(self.checkbox, expand=True, margin_right=10)

        # choices = [_("Default")]
        choices = []
        for val in values:
            if val >= 1024 * 1024:
                choices.append("{0:0.1f} GB".format(val / (1024 * 1024)))
            elif val >= 10 * 1024:
                choices.append("{0:0.0f} MB".format(val / 1024))
            elif val >= 1024:
                choices.append("{0:0.1f} MB".format(val / 1024))
            else:
                choices.append("{0} KB".format(val))
        self.choice = fsui.Choice(self, choices)
        # self.layout.add_spacer(0, expand=True)
        self.layout.add(self.choice)

        self.initialize_from_config()
        self.set_config_handlers()

    def initialize_from_config(self):
        self.on_config(self.option_key, Config.get(self.option_key))

    def set_config_handlers(self):
        self.choice.on_change = self.on_choice_change
        Config.add_listener(self)

    def on_destroy(self):
        Config.remove_listener(self)

    def get_default_value(self):
        return Amiga.get_default_option_value(Config.get("amiga_model"),
                                              self.option_key)

    def on_checkbox_change(self):
        print("\n\n\n\non_checkbox_change\n\n\n\n")
        if self.checkbox.is_checked():
            if Config.get(self.option_key) == "":
                print("-- setting config to default value")
                Config.set(self.option_key, self.get_default_value())
            # self.choice.enable()
            # self.select_value(Config.get(self.option_key))
        else:
            # use default value
            Config.set(self.option_key, "")
            # self.choice.disable()
            # self.select_value(self.get_default_value())

    # def update_ui(self):
    #     print("\n\n\n\n -- update_ui -- \n\n\n\n")
    #     if self.checkbox.is_checked():
    #         if Config.get(self.option_key) == "":
    #             print("-- setting config to default value")
    #             Config.set(self.option_key, self.get_default_value())
    #         self.choice.enable()
    #         self.select_value(Config.get(self.option_key))
    #     else:
    #         # use default value
    #         Config.set(self.option_key, "")
    #         self.choice.disable()
    #         self.select_value(self.get_default_value())

    def on_choice_change(self):
        print("\n\n\n\n -- on_choice_change -- \n\n\n\n")
        index = self.choice.get_index()
        # if index == 0:
        #     value = ""
        # else:
        # value = str(self.values[index - 1])
        value = str(self.values[index])
        Config.set(self.option_key, value)

    def on_config(self, key, value):
        if key == "amiga_model":
            # self.update_ui()
            self.on_config(self.option_key, Config.get(self.option_key))
        elif key == self.option_key:
            print("-----", key, value)
            if value:
                if not self.checkbox.is_checked():
                    self.checkbox.check(True)
                # self.update_ui()
                if not self.select_value(value):
                    print("WARNING: could not find memory setting")
                self.choice.enable()
            else:
                #
                if self.checkbox.is_checked():
                    self.checkbox.check(False)
                self.select_value(self.get_default_value())
                self.choice.disable()
                # self.update_ui()
                # self.choice.set_index(0)

    def select_value(self, value):
        for i, config in enumerate(self.values):
            if str(config) == value:
                # self.choice.set_index(i + 1)
                self.choice.set_index(i)
                return True
        return False



"""
This module contains information about files present on common Workbench
disks. The data is used when creating a Workbench environment on demand.
Some files are removed because they differ between Cloanto WB floppies and
original WB floppies.
"""
import os
import sys
import hashlib
import traceback
from fsgs.amiga.adf import ADFFile


wb_133_startup_sequence = """\
c:SetPatch >NIL: ;patch system functions
Addbuffers df0: 10
cd c:
echo "A500/A2000 Workbench disk.  Release 1.3.3 version 34.34*N"
Sys:System/FastMemFirst ; move C00000 memory to last in list
BindDrivers
SetClock load ;load system time from real time clock (A1000 owners should
              ;replace the SetClock load with Date
FF >NIL: -0 ;speed up Text
resident CLI L:Shell-Seg SYSTEM pure add; activate Shell
resident c:Execute pure
mount newcon:
;
failat 11
run execute s:StartupII ;This lets resident be used for rest of script
wait >NIL: 5 mins ;wait for StartupII to complete (will signal when done)
;
SYS:System/SetMap usa1 ;Activate the ()/* on keypad
path ram: c: sys:utilities sys:system s: sys:prefs add ;set path for Workbench
LoadWB delay  ;wait for inhibit to end before continuing
endcli >NIL:
"""


wb_204_startup_sequence = """\
c:setpatch >NIL:
c:version >NIL:
addbuffers >NIL: df0: 15
Failat 21

Resident >NIL: C:Execute PURE ADD

makedir ram:T ram:Clipboards ram:env ram:env/sys
copy >NIL: ENVARC: ram:env all quiet noreq

assign ENV: ram:env
assign T: ram:t ;set up T: directory for scripts
assign CLIPS: ram:clipboards
assign REXX: s:

if exists sys:Monitors
    join >NIL: sys:monitors/~(#?.info) as t:mon-start
    execute t:mon-start
    delete >NIL: t:mon-start
endif

BindDrivers

setenv Workbench $Workbench
setenv Kickstart $Kickstart

IPrefs

echo "Amiga Release 2.  Kickstart $Kickstart, Workbench $Workbench"

conclip

mount speak:
mount aux:
mount pipe:

path ram: c: sys:utilities sys:rexxc sys:system s: sys:prefs sys:wbstartup add
if exists sys:tools
    path sys:tools add
    if exists sys:tools/commodities
        path sys:tools/commodities add
    endif
endif

; If this is the initial boot (i.e. keyboard env variable is not set)
; then execute PickMap which will query for a keymap and set the
; keyboard env variable.
;
; if keyboard env variable is set, set the keymap
if ${sys/keyboard} NOT EQ "*${sys/keyboard}"
    setmap ${sys/keyboard}
else
    PickMap sys: initial
endif

if exists s:user-startup
    execute s:user-startup
endif

LoadWB

endcli >NIL:
"""


wb_300_startup_sequence = """\
; $VER: startup-sequence 39.9 (9.8.92)

C:SetPatch QUIET
C:Version >NIL:
C:AddBuffers >NIL: DF0: 15
FailAt 21

C:MakeDir RAM:T RAM:Clipboards RAM:ENV RAM:ENV/Sys
C:Copy >NIL: ENVARC: RAM:ENV ALL NOREQ

Resident >NIL: C:Assign PURE
Resident >NIL: C:Execute PURE

Assign >NIL: ENV: RAM:ENV
Assign >NIL: T: RAM:T
Assign >NIL: CLIPS: RAM:Clipboards
Assign >NIL: REXX: S:
Assign >NIL: PRINTERS: DEVS:Printers
Assign >NIL: KEYMAPS: DEVS:Keymaps
Assign >NIL: LOCALE: SYS:Locale
Assign >NIL: LIBS: SYS:Classes ADD
Assign >NIL: HELP: LOCALE:Help DEFER

IF NOT EXISTS SYS:Fonts
  Assign FONTS:
EndIF

BindDrivers
C:Mount >NIL: DEVS:DOSDrivers/~(#?.info)

IF EXISTS DEVS:Monitors
  IF EXISTS DEVS:Monitors/VGAOnly
    DEVS:Monitors/VGAOnly
  EndIF

  C:List >NIL: DEVS:Monitors/~(#?.info|VGAOnly) TO T:M LFORMAT "DEVS:Monitors/%s"
  Execute T:M
  C:Delete >NIL: T:M
EndIF

SetEnv Workbench $Workbench
SetEnv Kickstart $Kickstart
UnSet Workbench
UnSet Kickstart

C:IPrefs

C:ConClip

Path >NIL: RAM: C: SYS:Utilities SYS:Rexxc SYS:System S: SYS:Prefs SYS:WBStartup SYS:Tools SYS:Tools/Commodities

IF EXISTS S:User-Startup
  Execute S:User-Startup
EndIF

Resident Execute REMOVE
Resident Assign REMOVE

C:LoadWB
EndCLI >NIL:
"""


wb_133_files = {
    "c/": "",
    "c/AddBuffers": "b4935ca3f55fbf53db41a61d32cee48d93d1c431",
    "c/Ask": "20156be306fadfe45b8fde83bbc90258f371a00c",
    "c/Assign": "52579fe27010466442e928cb2a62cecc20f9f184",
    "c/Avail": "9d3d8ff67cfc4d7e220db53268830b0a5b5f0a2b",
    "c/Binddrivers": "0f7eaf913a9a8d50af3573a695a8f4af3b8f0ad6",
    "c/Break": "7c1cb7e382d36feb70b773b6cfd030ff853057e1",
    "c/CD": "9f4c181e7d5e3c0e103f462ba1420570499ca922",
    "c/ChangeTaskPri": "f63a5652aacd198b6ee25a2aefa3518a53f14ab6",
    "c/Copy": "ef236641b73d0b849f6ceb4a218469ea8f1d06f2",
    "c/Date": "287859f1c0fc0f1772c9e9c4f767bd523515b194",
    "c/Delete": "d6304c0e704c94073ad67edd19725a4742697ef6",
    "c/Dir": "c4c80a92749862fe8fe877dff0d7ab88a824026d",
    "c/DiskChange": "37665418b614fb1d768ea85c32a165cfaec146c1",
    # "c/DiskDoctor": "c801fbdcc4b04ff492a67cfeaee0036fce70bb85",
    "c/Echo": "b43fc80f2fb23af7c3ee14c8e1b5eecfc3c587ee",
    # "c/Ed": "dad70eb4a65febb74283c2a189d6bb95b22b0886",
    "c/Edit": "d83ee8a2e1f9d1697875366dafc329f8e8e4d7e9",
    "c/Else": "8f64d3d3fc7c84e02c4211e68ca74b2fa528df7b",
    "c/EndCLI": "24a5167194970a0188c00b234a12d2fba4308478",
    "c/EndIf": "7f15af4934c472489f278268667c2788c2a7021b",
    "c/EndSkip": "7f15af4934c472489f278268667c2788c2a7021b",
    "c/Eval": "5af8c120e9cf1399740dab3eca367af242d015cc",
    "c/Execute": "29b7093beae26deab1bd66a98167bfdb96aa7cf7",
    "c/Failat": "49918ba167529cef1320452d7442eac73712e0e7",
    "c/Fault": "e7f20bbe32843bca9c206cab8be50ae1d77b680f",
    # "c/FF": "dc9b9750c2a38546f7be2dbe983ae258f2496b98",
    "c/FileNote": "36d14d7468eb136b1179f04fa3c218832638e960",
    "c/GetEnv": "be841e3d1794a11b9af1b299ff1d8c9be85d8338",
    "c/IconX": "442df04019ae6ff426c3464f8be8c523ad292830",
    "c/If": "e0f0c4880f33ab782d903663535812f777362983",
    "c/Info": "a22a7a4aa07bad698b725253bd796c6fbdca9eb8",
    "c/Install": "1792a60e0564098429838d8db592b6297769d6f9",
    "c/Join": "0e8d276bb1f70a52c608cd8c8315ef8e84cceaef",
    "c/Lab": "7f15af4934c472489f278268667c2788c2a7021b",
    "c/List": "0d5c1cddab2af3e41bfe485e4a8cb76dc99c4bc4",
    "c/LoadWB": "13f61f0d60028256cc790f4cef3bcca83fb20259",
    "c/Lock": "f8a6d2e97b637c1ac1be97eaf77e3f12c79154ac",
    "c/Makedir": "d01c425e8af0f4f4084a6b2ee13780240db4f027",
    "c/Mount": "4f18b1c83c0f7a59eaa11c08e7a82cb7941740b3",
    "c/NewCLI": "9298e77f0adaa7a789d6e5f9f3035f24e60364d6",
    "c/NewShell": "7b827b5fa5456c9e1d722f2611437864be1bfdec",
    "c/Path": "770f775d73fdd8df8c1890be2ba2f9fbd388c06b",
    "c/Prompt": "5bb558e3f2ed62d5f676e84bc23f70ed913b4412",
    "c/Protect": "fec1b09d6a808b0e101ee2ed923a7f0e60597caa",
    "c/Quit": "e8f9244859b2e55e5c10304b1ca35e5801265c23",
    "c/Relabel": "496bc49e1488e8276e2074063b839269b34d1889",
    "c/RemRAD": "bca2dbf0ea2a8885fd976c04d421b884796d070a",
    "c/Rename": "b94baf0954756f75272576c7b34cdd6859d896d3",
    "c/Resident": "70c21c5a6add12526b29690f8bde0327f91b6f78",
    "c/Run": "caf92a05af573c54a1c9044e51ac6aeaab534e23",
    "c/Search": "0359aa4849bb1e7f7fa37862a4ab2f34b6d3b6fb",
    # "c/SetClock": "95d98f533207f53c1d90ddc57f2815db56d9bba7",
    "c/SetDate": "cca8245571bd7ffc4ddd46e87441b2600bf8be74",
    "c/SetEnv": "ac8df5ae6adfc7d68e5a3f514b935bd5293b09b7",
    "c/SetPatch": "3b36ee49e10626594545c564957fb4882ac2bfc3",
    "c/Skip": "15c513c48b568dc77593c13bf593fc6f8e0bcdb8",
    "c/Sort": "39f4a8a41ec5ab3cb030be579b49b6a3a7046f5a",
    "c/Stack": "d48dc30db5f9d6c7c04259444ba74b7c2f3dffd0",
    "c/Status": "08fffe1acc5dec49345e188eaba1f2fa74cd7424",
    "c/Type": "afb53409c675bac58048e8e76d403d61e91c1fc9",
    "c/Version": "bfa3c28746dc9a56df4569e687d0c7f6bb7e5b80",
    "c/Wait": "c893b36bd2376c003e3d2b2f5791d757c5fa33f9",
    "c/Which": "d1c3a0b2d9a0bcc3db3580182b75961981e9d38d",
    "c/Why": "84d318064b8374399bad8139c5f8f6046aa0b6e0",
    "devs/": "",
    "devs/clipboard.device": "d1708898f40bdb4f525e615e833e7f109331ac56",
    "devs/clipboards/": "",
    "devs/keymaps/": "",
    "devs/keymaps/usa1": "39d79e8a775260c13df0f657ee20e5f3e742de9e",
    "devs/MountList": "b85692e12b2fc7eaa02e4d211b043b5cd6272200",
    "devs/narrator.device": "d51631238dc07925b89e57ebe9d2e412400a0384",
    "devs/parallel.device": "861b780af555f52088adf5106bb62c05444dba82",
    "devs/printer.device": "5f06ec8cd7c32c40d78af95c8953ec527647a861",
    "devs/printers/": "",
    "devs/printers/generic": "aaaab28025bef46e5c01dde3d01d2f03fc63df2b",
    "devs/ramdrive.device": "ca60b73bc779d95cdcf23ec5e92b44dc21b1b472",
    "devs/serial.device": "55df2ec0a1d9dc6a03c9335737e23a38b2d73213",
    "devs/system-configuration": "f4e0ca14cbfb69c78c491d4cb4253a9a8ba9a78b",
    "Empty/": "",
    "Expansion/": "",
    "fonts/": "",
    "fonts/diamond.font": "cfe171bc8ab48615c6ac8fdfc5cd9da098f35e66",
    "fonts/diamond/": "",
    "fonts/diamond/12": "e6634c30e031dae19428423952f0abbe75d84559",
    "fonts/diamond/20": "75cb8014065834d1be7261a265bf0939e95d483a",
    "fonts/emerald.font": "98d935aa47ca406384bb6c592c63549f2d43d604",
    "fonts/emerald/": "",
    "fonts/emerald/17": "359d8c7da2e5ff4c157eed528c8c41a5ea09530a",
    "fonts/emerald/20": "de377bd498532ff4d3aa92e60fae8a029e7fc608",
    "fonts/garnet.font": "57e054da85337f25d660d2eb983dcf924a44ca4d",
    "fonts/garnet/": "",
    "fonts/garnet/16": "e99e9fde239b557ba181d91261f84abfda6f8bcb",
    "fonts/garnet/9": "5705e5553c85dc2e6573e4420d704e9fca5efe25",
    "fonts/opal.font": "dc66cd2d713f027b22539c503c5a5647e1f29fbf",
    "fonts/opal/": "",
    "fonts/opal/12": "ebb7b174ba5f215172921ec7e8cb3774eb21c90b",
    "fonts/opal/9": "0cea9720d43e2280b6fa8583fcefa364b9540446",
    "fonts/ruby.font": "314e2c9ca75cb75cadca1038d19a5ab92f2ed656",
    "fonts/ruby/": "",
    "fonts/ruby/12": "c5b1fd7d3834ebd2a26fc23c42f174451ddea1c7",
    "fonts/ruby/15": "58e3f44c65d75c9bff0a9992536c084c2572dbc9",
    "fonts/ruby/8": "a09b9ce5bd1c4a68773e6abfccc5218d0ddd066d",
    "fonts/sapphire.font": "18f992c2fef46fd408b272f07f9d5598c5e61bcb",
    "fonts/sapphire/": "",
    "fonts/sapphire/14": "fb0da752206e8b29e94f6e3d68492c2f794c54d5",
    "fonts/sapphire/19": "250e989ac01401b10773ad11cf9e45ad83437fe6",
    "fonts/topaz.font": "66b2ef88f256216f340a2130920c3dafdc5b3038",
    "fonts/topaz/": "",
    "fonts/topaz/11": "be925d2d0a61962ff1de9deb742747a356fc2b87",
    "l/": "",
    "l/Aux-Handler": "2e621414176ce93959e066cacc86f16c70bb7f80",
    "l/Disk-Validator": "30ede410472723c95338ed2c870916ad275e9706",
    "l/FastFileSystem": "cc14a3ce4805d4e3a0397fe6997ff5428d90ecdf",
    "l/Newcon-Handler": "c530e2ce63f2c6203526345fb2d434f2d4a9f49d",
    "l/Pipe-Handler": "4e73c34bcf73ab5b590c7b016dc1176673322077",
    "l/Port-Handler": "d7e4ddab8cbd94f751a6ffd4bc93cfe07cb4cae2",
    "l/Ram-Handler": "c0b036019fbffc46b417a72dd296f8d33b19a308",
    "l/Shell-Seg": "4bbe3ccb55fab49c83ff7ab52b2af277e6d92429",
    "l/Speak-Handler": "1b3b3dcd0b46cc6dbf98467e319d0d396e7df22f",
    "libs/": "",
    "libs/diskfont.library": "f1a4d45d97f2df1cff6b83a4392c34cc7ac18bbc",
    "libs/icon.library": "4c9b95e6f786f707aa51b2483a9f21ac5e66d7c9",
    "libs/info.library": "aa46c186d762dc68d2e799a5d13cad6745b55d10",
    "libs/mathieeedoubbas.library": "cf565b2b7a9f6e8aae030e32490346589f90b780",
    "libs/mathieeedoubtrans.library":
    "29f906dc50594aca3a91cfb69bd3f332fdb1671f",
    "libs/mathtrans.library": "65bae4aac804fb4db79a427f12b6ef1e16d22882",
    "libs/translator.library": "6ddc4462f261317a61f3c371091021b243df562c",
    "libs/version.library": "8f062571cac294bf5556b5bdc9d48da2c2f21fae",
    "Prefs/": "",
    "Prefs/CopyPrefs": "0af96045315193d4ec4f6c40d5ea6c3765d3fb04",
    "Prefs/Preferences": "e8b98bbb4cc3a04cf31855d4dcf6c292d02a48b4",
    "s/": "",
    "s/CLI-Startup": "c9abb720ab20b5976e18c45dd6eee46187b367f9",
    "s/DPAT": "bd0ee7066b43ef7df98db2801a75ab4f79f13098",
    "s/PCD": "d6ff57db9f9cd193d99aa64581ca964494b5e430",
    # "s/Shell-Startup": "25ee071ea6069e7fb10ac23bef1bbb24426b1bf1",
    "s/SPAT": "208cd7d832fb8a0d81d45fc3db7bc31c12727b3f",
    "s/Startup-Sequence.HD": "0d3caa59e5f867a6f9f8c68d318a16da1bf60fd6",
    # "s/StartupII": "dae0c0aeb988f27c5a9a85dcdf502998570b3b45",
    "Shell": "da39a3ee5e6b4b0d3255bfef95601890afd80709",
    "System/": "",
    "System/CLI": "a6e7eb1585e4ebd6f4ed606eb18d8f23af184bea",
    "System/Diskcopy": "009cfa0952888c059da450b46d731d7f1779ae63",
    "System/FastMemFirst": "5fd51854d5b109fcdbf52809497e642036aa4722",
    "System/FixFonts": "3fc2dfbc62326678c3daf9ee860bac6330666999",
    # "System/Format": "972bbf474d471bec395d19f6d737908542055d70",
    "System/InitPrinter": "6d35a5215846acef053211dfcd6563054b51da10",
    "System/MergeMem": "04c018c68442187d292c0a57e1686bc8269f2ac4",
    "System/NoFastMem": "87e8c332e69a5446535923157cb6678ccb992614",
    "System/SetMap": "6d6c0ca1163c9d921b193d0e3c04722800c8c88d",
    "t/": "",
    "Trashcan/": "",
    "Utilities/": "",
    "Utilities/Calculator": "66d0b0fcb4795517c6d30cb96662afce68d81585",
    "Utilities/Clock": "ca791b988dd647432dd36dcc519bc5ce7d2e0bd2",
    "Utilities/ClockPtr": "47c191dd0edeccbf80cb5402002101bcc791e171",
    "Utilities/CMD": "7e0917b6fbf4152fd9167af0eee2118b35d48fe7",
    "Utilities/GraphicDump": "b82aa41aa1d602a1e1452d757797dd22b97dc75a",
    "Utilities/InstallPrinter": "05f57e5ffc717c7a4eb80bb3917be6006ebdc0b0",
    "Utilities/More": "2eb6ab87d1ee9bf28bea826e2481dd3c4f6c1447",
    "Utilities/Notepad": "ca9e25cd122dc01a28b66eb8f3ebc3395504e6b7",
    "Utilities/PrintFiles": "66d70dae78836d1aa198b9fd46492f204be0459f",
    "Utilities/Say": "3f845bf4d06c680c960847115449adb4819cd8b7",
}


wb_204_files = {
    "C/": "",
    "C/AddBuffers": "6ed1a3f24a1f605aa7234410faebbfbb3fda3ee6",
    "C/Assign": "b5b7edb67f578019d46af425a16458ec0cdb1c2e",
    "C/Avail": "5e62c202fe3cb447a19c39efb43c935eed139a66",
    "C/BindDrivers": "7534362e5def2eb32819be52f242d28ebdc05a26",
    "C/Break": "97ca2404e17fc78e3d988131bb17183c5c3aa15a",
    "C/ChangeTaskPri": "66533bb2cee36475b03cd73ae8ab198889e81be3",
    "C/ConClip": "96dd0671b40743c18f9ec2b58b2ccb731092e29f",
    "C/Copy": "628a17ad2b883565a1312d105a65764076c42013",
    "C/CPU": "40bcedc1af7677096cb1e5f5e41e147cf4c952fc",
    "C/Date": "0d190c0ce0b99cd653a8f982964eed110ac540eb",
    "C/Delete": "0597fd916dd28850ef68bb904937cad0d347cf5a",
    "C/Dir": "184eef562670fd6b22b3025e385f992469fa50d4",
    "C/DiskChange": "2cae12bfbb1cc98348e7afabf0f48b09cadbaf5e",
    "C/DiskDoctor": "c801fbdcc4b04ff492a67cfeaee0036fce70bb85",
    "C/Ed": "077483aacde3b40eeeb7b2b08cff3c18d6174e1c",
    "C/Edit": "14a1b22e4d72f85cd64b49cfdc8380b294f2da15",
    "C/Eval": "fae9792514d07b6fbb5b04d778492941dee5c448",
    "C/Execute": "ef1e6f6db0f3a513e0dc98add537e1db87b1856a",
    "C/Filenote": "6c263e034a3aca677467d410e410ba808a5f2b8d",
    "C/IconX": "9a2088f98175b3341f8bf4831599d54832430c35",
    "C/Info": "d12eea32aef864fcb4717ee8447c01d5900c94f1",
    "C/Install": "b5e1a2ea287c29390d38e47ccb3749f7c449bd4e",
    "C/IPrefs": "f3eb8607aaac99043020f0a9a1156dec4f941a11",
    "C/Join": "da8d787eadd16e682216c808cbbd1427acde7280",
    "C/List": "74a07847e45f8ae61daa029dd21775f438013a08",
    "C/LoadWB": "c705d1579c04505819a6bc3c6c29ae8d2610da66",
    "C/Lock": "a671aabf698196baa6a237601d07fb49b7f64f60",
    "C/MagTape": "8d14c533f6fb73286a642bb208151c36da3af2c9",
    "C/MakeDir": "3e00b210538357ff2ca9f0bc16471f06fb994282",
    "C/MakeLink": "da8472a553ece5484d7ba1290884ad5d59db5770",
    "C/Mount": "b2f1e59e977cec1c83166977cd4350f5388c3c9f",
    "C/Protect": "e95073c0169dd7c7ca39b9842b339a1288ca3016",
    "C/Relabel": "056cb6a12033f5638b1353be1007106fd9267a6e",
    "C/RemRAD": "7e5d53c865debaa9d752f96ef818320248332ef6",
    "C/Rename": "d1cb2ab35c8fa0dc912b98dbcf002c2c3b220988",
    "C/Search": "89c1dd049ca2fff58269fd44af2705abdedff112",
    "C/SetClock": "0c166dd6f8baa45eda766f3b41a0f799f8e7f388",
    "C/SetDate": "d7a78803ade6e09dbbba051ab6d4fbcb923527eb",
    "C/SetFont": "8b5bd918009242703f5f8ae9237199bd35ef0efe",
    "C/SetPatch": "309db203c5c88d2867d9d6f75e5ef67d2c558a95",
    "C/Sort": "d886f81bdc74f4942acb52680e56d8bf50a0be8a",
    "C/Status": "71e09cfc79d8ede0721ba4a01e1cd43a6318a99f",
    "C/Type": "00917f88e78770f6b28562bede2cf64718325553",
    "C/Version": "86dcf19836869c7319a1b89083a0529d1ee0d049",
    "C/Wait": "a0d503a136dad028ec51e33cfd57d130c1b4b807",
    "C/Which": "22952cce8446a1405b1b22d057ec19614e38c366",
    "Devs/": "",
    "Devs/clipboard.device": "83f3490b6480a0515b37e6f926ab48daf9cad61e",
    "Devs/Keymaps/": "",
    "Devs/MountList": "b32b62007cbfd4e5412f003f3fd83a183845b447",
    # "Devs/narrator.device": "c5a35d605c39b9c59542d71883238a02e92d726d",
    "Devs/parallel.device": "88c8c2bae625caf7b2bdbea9688a763f035bc199",
    "Devs/printer.device": "b13d1ec922deafa5a479fc1c591f911ff95559eb",
    "Devs/Printers/": "",
    "Devs/Printers/generic": "11473c5271b970e36c70b408aac7c67fff7d97b6",
    "Devs/serial.device": "d814d6ce8efb4a87966dae581eddf4dca528094a",
    "Devs/system-configuration": "4be3d7e8395a5827085f57cb6cdbf5e88fa78506",
    "Expansion/": "",
    "Fonts/": "",
    "L/": "",
    "L/aux-handler": "5a20c44cdcba0793fd292e3f951832ad4670f65e",
    "L/port-handler": "d7e4ddab8cbd94f751a6ffd4bc93cfe07cb4cae2",
    "L/queue-handler": "a458c6935c90d8a9422600c84875237e0558f89b",
    # "L/speak-handler": "d577708e1a0be7566885824349102025f1a250c4",
    "Libs/": "",
    "Libs/asl.library": "8d1fd81d7c128c397443f0ceb696dab3fecc5828",
    "Libs/commodities.library": "eaa02d69480d8df876f3932bf13f7c6e6ebc6c78",
    "Libs/diskfont.library": "97022049498794fe8f6135b53ed01d0688903499",
    "Libs/iffparse.library": "32eb189c8c003e8bdb1c836fa0ffd12a5d2c9f17",
    "Libs/mathieeedoubbas.library": "ce7888086d9749d738e1141e7b7e200f5eb780a9",
    "Libs/mathieeedoubtrans.library":
    "6d6e29a25f7bc87d26a56d437441d3a2c789b126",
    "Libs/mathieeesingtrans.library":
    "b9b164b6a7bff61ffd703999c93f756ea447452f",
    "Libs/mathtrans.library": "92d5888b3d2d3bb83c66cc6e486d303821c518c9",
    "Libs/rexxsupport.library": "7ae7acdd99a89d00b706060f818ad340869827a2",
    "Libs/rexxsyslib.library": "b74995c09a0d6350498579b8ff406477ae5b9566",
    # "Libs/translator.library": "e99c035faf5184a3397c89a3df1a7606c8417be6",
    "Libs/version.library": "ac699c82157ccc204aaf1eb78e3c53c4a8f13bf5",
    "Monitors/": "",
    "Prefs/": "",
    "Prefs/Env-Archive/": "",
    "Prefs/Env-Archive/sys/": "",
    "Prefs/Env-Archive/sys/wbconfig.prefs":
    "9658314fdb2a32286dba8830cf701469ac0089d3",
    "Prefs/Font": "c8d0f7bd565fb151ff0738e9df9f063cd43cf244",
    "Prefs/IControl": "c3d984f4ca7213194c8bfa5cb20ef8487c107e7a",
    "Prefs/Input": "37b19e8dd679c1d901c6620d0635cd9a56b92ab9",
    "Prefs/Overscan": "d15843c3147bff57f3dd4ffc2303d1609ca0eb12",
    "Prefs/Palette": "3d8712e5b337e8a02c5e786975b5724bd9716f8e",
    "Prefs/Pointer": "79c013f9d2065b7c2f824e530c05e65b182c00de",
    "Prefs/Presets/": "",
    "Prefs/Printer": "89a003a1b26c5a0b9eb2fdf6ddd53cb5368445cb",
    "Prefs/PrinterGfx": "84d71c99acac85cf65cf073e736b55871e286d2c",
    "Prefs/ScreenMode": "640adc69430a72f08a42761b4d3740840d95a834",
    "Prefs/Serial": "8b7e24eb43396dc0fc5a341a2425213fb7d77ac3",
    "Prefs/Time": "bb07a47dbb7453fbeb61ecae5ba9459f33b4217c",
    "Prefs/WBPattern": "42b809a0d4f02ea5e7f720df3e7a5e97827a1689",
    "Rexxc/": "",
    "Rexxc/HI": "d3934c81b5a7832f0cdd64650f7c74eacc608a0a",
    "Rexxc/RX": "527adec943412976b2d40c4556ca48e0a5b0abee",
    "Rexxc/RXC": "b42afa06c53df3221683a2d42b2b2ce4f38a4525",
    "Rexxc/RXLIB": "04b10ce3d05feb2fdbd722e5a275a0c844e7d86e",
    "Rexxc/RXSET": "702b4613236e8faba2270744229171dc46a0d5ed",
    "Rexxc/TCC": "473c7136e636b9505332118cd69a3ba29db0dbbe",
    "Rexxc/TCO": "f8b901a6ec8c6844a7205f5ceec5aad6e6261531",
    "Rexxc/TE": "2f89c6cd66e559b6f3cf5a0783b14fb18236931d",
    "Rexxc/TS": "45a5aa31dd4d42099d41948ef976599fa221938d",
    "Rexxc/WaitForPort": "f3fea65223cb028f479d1831237385812675a065",
    "S/": "",
    "S/BRUtab": "36cad75345610b07e447edb5000a368ce14de0df",
    "S/DPat": "4c8aac6a4989201c01d36247a9622288beb6d291",
    "S/Ed-startup": "298221ea95f4bfa2cc9b0cec4d8643f708a16abc",
    "S/HDBackup.config": "5284e3e2897148a373da769df16530cd1fff45c6",
    "S/PCD": "d6ff57db9f9cd193d99aa64581ca964494b5e430",
    "S/PickMap": "0b9ca76ba793031e11dfd14cf26b1257f9b24d89",
    "S/Shell-startup": "3d50527c6c17256f50d04d212d97d0fb27bf862f",
    "S/SPat": "870ac2f50dbb2a75989c5418a2012df81455bea8",
    "S/Startup-sequence.HD": "d66c044a25bc4235ec1d2479f368aca733a4d77d",
    "System/": "",
    "System/AddMonitor": "9ef54a726a70298608654a8be35388adbbd4a5eb",
    "System/BindMonitor": "5bafd3b20970c9cb05e9619f28091d56ad1dc2c8",
    "System/CLI": "1d22d9100a26d12d375fe1fd938f46b5c09017f1",
    "System/DiskCopy": "c91ae2a853c386c6c4c2c5dbed394bc69074f552",
    "System/FixFonts": "5f093b674d951599af5b5b2f82f3007cfdfa2902",
    "System/Format": "e8f87083aefbaf68f03947accca10278b95ad9d5",
    "System/NoFastMem": "abb2cff5c39351d111bce07c22585d58219b31a1",
    "System/RexxMast": "6f64f8bc51aec68344fae4151697976f59946b8e",
    "System/Setmap": "859abaaab6de713c2a46043f5f5421d54b79d88f",
    "t/": "",
    "Trashcan/": "",
    "Utilities/": "",
    "Utilities/Clock": "079dadb61258bbb07112a4109b58552fe61f5741",
    "Utilities/Display": "5c21232ceea79568d915d2d9d0a96e80e01fd4cb",
    "Utilities/Exchange": "b227f5823b2b07f92670f2628426645fbfa4ec40",
    "Utilities/More": "39d44f82ad645b9b1cd2931125c5b340613f7e29",
    "Utilities/Say": "79614ec17080d87d580f6cd1e20e74f53c1b5174",
    "WBStartup/": "",
}


wb_300_files = {
    "C/": "",
    "C/AddBuffers": "6ed1a3f24a1f605aa7234410faebbfbb3fda3ee6",
    "C/AddDataTypes": "e8bbd0b50fb5de6fa2382e845e1d67011a1ddd2a",
    "C/Assign": "b5b7edb67f578019d46af425a16458ec0cdb1c2e",
    "C/Avail": "5e62c202fe3cb447a19c39efb43c935eed139a66",
    "C/BindDrivers": "be270c04b4591f10da960231e488929d67e58135",
    "C/Break": "97ca2404e17fc78e3d988131bb17183c5c3aa15a",
    "C/ChangeTaskPri": "66533bb2cee36475b03cd73ae8ab198889e81be3",
    "C/ConClip": "956fd130af649ac441d4b68ea9b01e29d4861357",
    "C/Copy": "0c5f9470fbcb36fdbcdf970fd1b22aa627328a24",
    "C/CPU": "0191712ede6348b8e1803f5807a941a85c12bd0a",
    "C/Date": "0d190c0ce0b99cd653a8f982964eed110ac540eb",
    "C/Delete": "0597fd916dd28850ef68bb904937cad0d347cf5a",
    "C/Dir": "184eef562670fd6b22b3025e385f992469fa50d4",
    "C/DiskChange": "2cae12bfbb1cc98348e7afabf0f48b09cadbaf5e",
    "C/Ed": "077483aacde3b40eeeb7b2b08cff3c18d6174e1c",
    "C/Edit": "14a1b22e4d72f85cd64b49cfdc8380b294f2da15",
    "C/Eval": "fae9792514d07b6fbb5b04d778492941dee5c448",
    "C/Execute": "ef1e6f6db0f3a513e0dc98add537e1db87b1856a",
    "C/Filenote": "6c263e034a3aca677467d410e410ba808a5f2b8d",
    "C/IconX": "09bd52d33b700538e9bed493f245a3ffce944ad3",
    "C/Info": "3f5f37e405ca929cf0b4c6ac6abbf7f8d4f19892",
    "C/Install": "b4bf5e4ecda66c6a66e1780d1143f632e573ed3b",
    "C/IPrefs": "5b5b70ec7b06a6fa142d01372a4e0fce1bff5461",
    "C/Join": "da8d787eadd16e682216c808cbbd1427acde7280",
    "C/List": "86c0a28542939ea5e0d2075f9db337d93b139fb7",
    "C/LoadWB": "e663a715c6ad69a7d3882b4772cbdc9de835e791",
    "C/Lock": "a671aabf698196baa6a237601d07fb49b7f64f60",
    # "C/MagTape": "8d14c533f6fb73286a642bb208151c36da3af2c9",
    "C/MakeDir": "3e00b210538357ff2ca9f0bc16471f06fb994282",
    "C/MakeLink": "da8472a553ece5484d7ba1290884ad5d59db5770",
    "C/Mount": "c691df2ceb10c6dccd4f735dea0f07c199e09074",
    "C/Protect": "e95073c0169dd7c7ca39b9842b339a1288ca3016",
    "C/Relabel": "056cb6a12033f5638b1353be1007106fd9267a6e",
    "C/RemRAD": "7e5d53c865debaa9d752f96ef818320248332ef6",
    "C/Rename": "d1cb2ab35c8fa0dc912b98dbcf002c2c3b220988",
    "C/RequestChoice": "5198856e4ebc9c88e5799627aeceaa42866f6525",
    "C/RequestFile": "46a45ca9906b52a215161c2141193fe740f41d71",
    "C/Search": "89c1dd049ca2fff58269fd44af2705abdedff112",
    "C/SetClock": "0c166dd6f8baa45eda766f3b41a0f799f8e7f388",
    "C/SetDate": "d7a78803ade6e09dbbba051ab6d4fbcb923527eb",
    "C/SetFont": "3b2bb7bb70d84a230a4614dae34c064b7cfc315d",
    "C/SetKeyboard": "d32a3aed9f10fdcbb4335296be50f70f2f15438e",
    "C/SetPatch": "4d4aae988310b07726329e436b2250c0f769ddff",
    "C/Sort": "d886f81bdc74f4942acb52680e56d8bf50a0be8a",
    "C/Status": "71e09cfc79d8ede0721ba4a01e1cd43a6318a99f",
    "C/Type": "00917f88e78770f6b28562bede2cf64718325553",
    "C/Version": "5a698d6494fa50fd3faa7e532def17b6c561d217",
    "C/Wait": "a0d503a136dad028ec51e33cfd57d130c1b4b807",
    "C/Which": "22952cce8446a1405b1b22d057ec19614e38c366",
    "Classes/": "",
    "Classes/DataTypes/": "",
    "Classes/DataTypes/8svx.datatype":
    "dae1f0ff6171479b27c21d3e32f7d4e383f969ab",
    "Classes/DataTypes/amigaguide.datatype":
    "d53c3e6abd119a3ca7c15805642a1e6ec96d374c",
    "Classes/DataTypes/ascii.datatype":
    "6f54362a5ce75221a62e0f1fd3a5d23fabc0f2bd",
    "Classes/DataTypes/ilbm.datatype":
    "c8251ed31ad79aa71cf6d3c134815c8f38c70226",
    "Classes/DataTypes/picture.datatype":
    "8028e6782c4069c45ffc4b51cef493f8339a1be9",
    "Classes/DataTypes/sound.datatype":
    "e4fbdbc5e55ddc4635dcd2e76e5623217c4a41a2",
    "Classes/DataTypes/text.datatype":
    "e6b80a188123cb397f7a79a1b580206bb369f2e2",
    "Classes/Gadgets/": "",
    "Classes/Gadgets/colorwheel.gadget":
    "7bbfccb8fd5d68b2ae8aac226eff877e8ebc9734",
    "Classes/Gadgets/gradientslider.gadget":
    "a54f39e9098f2c58e892b9183b555fe0a8574191",
    "Devs/": "",
    "Devs/clipboard.device": "c8d85bd384ea5033d5474283e83c02ad6cdaf32b",
    "Devs/DataTypes/": "",
    "Devs/DataTypes/8SVX": "b9116bb7654e12f7d281d32ca333a0f283e15532",
    "Devs/DataTypes/AmigaGuide": "e89b0bf0d75585b8f27e40d711164ab9e2a0c222",
    "Devs/DataTypes/FTXT": "cee60b9550ace3d44ba1d214ed350b05587c3919",
    "Devs/DataTypes/ILBM": "1cf3598ca839c554e6a7acc333ddd8327f8ac5d3",
    "Devs/DOSDrivers/": "",
    "Devs/DOSDrivers/PIPE": "624dd68e9a2794bf5bd60e58c1b3679a1dfd273c",
    "Devs/Keymaps/": "",
    # "Devs/mfm.device": "63b150010e420f96304375badca126812daa2255",
    "Devs/Monitors/": "",
    "Devs/parallel.device": "88c8c2bae625caf7b2bdbea9688a763f035bc199",
    "Devs/postscript_init.ps": "3c0bc3408a4ac929936f647637a04f45d227cca0",
    "Devs/printer.device": "aaf3970e54bedaf84cd69268fc6cf3730c395ba1",
    "Devs/Printers/": "",
    "Devs/Printers/Generic": "11473c5271b970e36c70b408aac7c67fff7d97b6",
    "Devs/serial.device": "d814d6ce8efb4a87966dae581eddf4dca528094a",
    "Devs/system-configuration": "4be3d7e8395a5827085f57cb6cdbf5e88fa78506",
    "Expansion/": "",
    "L/": "",
    # "L/aux-handler": "5a20c44cdcba0793fd292e3f951832ad4670f65e",
    # "L/CrossDOSFileSystem": "9f05f997b3aa0c3a431bea96cc4bbc153ba48814",
    # "L/FileSystem_Trans/": "",
    # "L/FileSystem_Trans/DANSK.crossdos":
    # "3db3dbfdb9f6d874368f12152944b00099fd9943",
    # "L/FileSystem_Trans/INTL.crossdos":
    # "6d803e82923564dd2f12b68836b2356282edfd93",
    "L/port-handler": "d7e4ddab8cbd94f751a6ffd4bc93cfe07cb4cae2",
    "L/queue-handler": "a458c6935c90d8a9422600c84875237e0558f89b",
    "Libs/": "",
    "Libs/amigaguide.library": "42b1ea9b94f12f4ad5275bb29a8a49fb977f1910",
    "Libs/asl.library": "ae59765242d7d1fc022d819a980652463e72688a",
    "Libs/bullet.library": "efffa71e955805d543842cbd0c67e75b591fe567",
    "Libs/commodities.library": "e52e0a7e2fb653a6895fb1cf15d0b37aed18f909",
    "Libs/datatypes.library": "1ea18282089ae14620c1fc71e82592d52217ff2a",
    "Libs/diskfont.library": "15d9765f1b66c4a3068d11186ea5bf41a8c8ad3b",
    "Libs/iffparse.library": "946ef60b4ba9f63593dbadfbc447546d7ab9725c",
    "Libs/locale.library": "a12b91f662a1c527ff6411b43555bc7eaeb370b4",
    "Libs/mathieeedoubbas.library": "08d8508cdcb77ad421a6dd6f80721b727c09d96b",
    "Libs/mathieeedoubtrans.library":
    "6d6e29a25f7bc87d26a56d437441d3a2c789b126",
    "Libs/mathieeesingtrans.library":
    "b9b164b6a7bff61ffd703999c93f756ea447452f",
    "Libs/mathtrans.library": "92d5888b3d2d3bb83c66cc6e486d303821c518c9",
    "Libs/rexxsupport.library": "7ae7acdd99a89d00b706060f818ad340869827a2",
    "Libs/rexxsyslib.library": "b74995c09a0d6350498579b8ff406477ae5b9566",
    "Libs/version.library": "9af4fa21ce77ca97b4604ffb095a9b2488295c85",
    "Prefs/": "",
    "Prefs/Env-Archive/": "",
    "Prefs/Env-Archive/Sys/": "",
    "Prefs/Env-Archive/Sys/wbconfig.prefs":
    "9658314fdb2a32286dba8830cf701469ac0089d3",
    "Prefs/Presets/": "",
    "Rexxc/": "",
    "Rexxc/HI": "d3934c81b5a7832f0cdd64650f7c74eacc608a0a",
    "Rexxc/RX": "527adec943412976b2d40c4556ca48e0a5b0abee",
    "Rexxc/RXC": "b42afa06c53df3221683a2d42b2b2ce4f38a4525",
    "Rexxc/RXLIB": "04b10ce3d05feb2fdbd722e5a275a0c844e7d86e",
    "Rexxc/RXSET": "702b4613236e8faba2270744229171dc46a0d5ed",
    "Rexxc/TCC": "473c7136e636b9505332118cd69a3ba29db0dbbe",
    "Rexxc/TCO": "f8b901a6ec8c6844a7205f5ceec5aad6e6261531",
    "Rexxc/TE": "2f89c6cd66e559b6f3cf5a0783b14fb18236931d",
    "Rexxc/TS": "45a5aa31dd4d42099d41948ef976599fa221938d",
    "Rexxc/WaitForPort": "f3fea65223cb028f479d1831237385812675a065",
    "S/": "",
    # "S/DPat": "227c6ed4ae33850577a0b03e22b7dfd966b421c4",
    "S/Ed-startup": "2a4e2dd940726199aea3c5cab73dcc527b624fb8",
    # "S/PCD": "02d4b5292a9ea3b56d68ca7bba3d1df5055b3a25",
    "S/Shell-Startup": "d039a940c89ec6c30a1b3568bfd6afe626a28b6d",
    # "S/SPat": "4f63e17cb9ca68b5dc5eb37fd02508f4c5ae0761",
    "System/": "",
    "System/CLI": "9f25538fd0a6b134dd7a094350731cdc1e9234e3",
    "System/DiskCopy": "902aa22b272961911a279bba4aca1fd7ba9c537b",
    "System/FixFonts": "86757c1120ce2137e3c72024fa9de898de8a7b24",
    "System/Format": "76918338279f5c1a2b86405f39d9e9c8e9ac0981",
    "System/NoFastMem": "6295d9937d3eb3370af14bf510c9ad02f687bd4b",
    "System/RexxMast": "6f64f8bc51aec68344fae4151697976f59946b8e",
    "T/": "",
    "Utilities/": "",
    "Utilities/Clock": "fda1ad2f1bfb730d9c0dbda4b1141935b67f65ae",
    "Utilities/More": "daced6e93fce3bf91a659ea161e343b87233cc54",
    "Utilities/MultiView": "c8e7b9f35907e168a03f5b46c9890b4094077750",
    "WBStartup/": "",
}


wb_133_floppies = [
    # Workbench v1.3.3 rev 34.34 (1990)(Commodore)(A500-A2000)
    # (Disk 1 of 2)(Workbench).adf
    "86cb4e007f9fdcf6c5eda6adba3f60f188063875",
    # Workbench v1.3.3 rev 34.34 (1990)(Commodore)(A500-A2000)(Dk)
    # (Disk 1 of 2)(Workbench)[m].adf
    "8d8314faa3b5fbc472c11d5fc669358522c1d00b",
    # amiga-os-134-workbench.adf
    "42c5af6554212e9d381f7535c3951ee284e127b2",
]


wb_204_floppies = [
    # Workbench v2.04 rev 37.67 (1991)(Commodore)
    # (Disk 1 of 4)(Workbench).adf
    "8d5c0310a86f14fb3e6a1da001ceb50b9a592c51",
    # Workbench v2.04 rev 37.67 (1991)(Commodore)
    # (Disk 1 of 4)(Workbench)[m].adf
    "d9d8bc1964d9159b3669fadedcd140ead197b0b7",
    # Workbench v2.04 rev 37.67 (1991)(Commodore)
    # (Disk 1 of 4)(Workbench)[m4].adf
    "21a4363e236011f0173c393207a2225a0a3002b0",
    # Workbench v2.04 rev 37.67 (1991)(Commodore)
    # (Disk 1 of 4)(Workbench)[m2].adf
    "898c0c372c476d8410890388b81ca642cc0b381d",
    # Workbench v2.04 rev 37.67 (1991)(Commodore)
    # (Disk 1 of 4)(Workbench)[m3].adf
    "5913fa0fb6cfa74ae9c80870f6ab8a4289036788",
    # amiga-os-204-workbench.adf
    "898c0c372c476d8410890388b81ca642cc0b381d",
]


wb_300_floppies = [
    # Workbench v3.0 rev 39.29 (1992)(Commodore)(A1200-A4000)(M10)
    # (Disk 2 of 6)(Workbench)[!].adf
    "e663c92a9c88fa38d02bbb299bea8ce70c56b417",
    # Workbench v3.0 rev 39.29 (1992)(Commodore)(A1200-A4000)(M10)
    # (Disk 2 of 6)(Workbench)[m2].adf
    "cf2f24cf5f5065479476a38ec8f1016b1f746884",
    # Workbench v3.0 rev 39.29 (1992)(Commodore)(A1200-A4000)(M10)
    # (Disk 2 of 6)(Workbench)[m5].adf
    "4f4770caae5950eca4a2720e0424df052ced6a32",
    # Workbench v3.0 rev 39.29 (1992)(Commodore)(A1200-A4000)(M10)
    # (Disk 2 of 6)(Workbench)[a].adf
    "9496daa66e6b2f4ddde4fa2580bb3983a25e3cd2",
    # Workbench v3.0 rev 39.29 (1992)(Commodore)(A1200-A4000)(M10)
    # (Disk 2 of 6)(Workbench)[m3].adf
    "0e7f30223af254df0e2b91ea409f35c56d6164a6",
    # amiga-os-300-workbench.adf
    "4f4770caae5950eca4a2720e0424df052ced6a32",
]


def update_files():
    base_dir = os.path.expanduser("~/Documents/FS-UAE/Floppies/Workbench")
    # floppies = {}
    floppy_file_sets = {}
    files = {}
    for floppy in os.listdir(base_dir):
        if not floppy.lower().endswith(".adf"):
            continue
        path = os.path.join(base_dir, floppy)
        with open(path, "rb") as f:
            floppy_sha1 = hashlib.sha1(f.read()).hexdigest()
        sha1sums = set()
        try:
            adf = ADFFile(path)
        except Exception:
            print("error parsing", path)
            traceback.print_exc()
            continue
        names = adf.namelist()
        for name in names:
            if name.endswith("/"):
                continue
            data = adf.open(name, "r").read()
            sha1 = hashlib.sha1(data).hexdigest()
            sha1sums.add(sha1)
            # floppies[floppy.lower()] = os.path.join(base_dir, floppy)
            files.setdefault(sha1, set()).add((floppy_sha1, name))
        floppy_file_sets[(floppy, floppy_sha1)] = sha1sums

    interesting_files = set()
    for file_map in [wb_204_files, wb_300_files]:
        for name, sha1 in file_map.items():
            if sha1:
                interesting_files.add(sha1)

    # print("")
    # print("workbench_file_map = {")
    # for sha1 in sorted(interesting_files):
    #     print("    \"{0}\": [".format(sha1))
    #     #print("        \"{0}\",".format(floppy_sha1))
    #     for floppy_sha1, name in sorted(files[sha1]):
    #         #print("        \"{0}\",".format(floppy_sha1))
    #         print("        (\"{0}\", \"{1}\"),".format(floppy_sha1, name))
    #     print("    ],")
    # print("}")
    # print("")

    for name, file_map in [
            ("wb_133_floppies", wb_133_files),
            ("wb_204_floppies", wb_204_files),
            ("wb_300_floppies", wb_300_files)]:
        print("\n" + name + " = [")
        file_set = set([x for x in file_map.values() if x])
        for floppy_data, floppy_file_set in floppy_file_sets.items():
            floppy_name, floppy_sha1 = floppy_data
            if floppy_file_set.issuperset(file_set):
                print("    # {0}".format(floppy_name))
                print("    \"{0}\",".format(floppy_sha1))
        print("]")


def main():
    if sys.argv[1] == "--update-files":
        update_files()
        return

    base_dir = os.path.expanduser("~/Documents/FS-UAE/Floppies/Workbench")
    floppies = {}
    for floppy in os.listdir(base_dir):
        if not floppy.lower().endswith(".adf"):
            continue
        floppies[floppy.lower()] = os.path.join(base_dir, floppy)

    floppy = floppies[sys.argv[1].lower()]
    adf = ADFFile(floppy)
    names = adf.namelist()
    startup_sequence = ""
    for name in names:
        if name.lower() == "s/startup-sequence":
            startup_sequence = adf.open(name, "r").read()
            continue
        if name.endswith(".info"):
            continue
        if name.endswith("/"):
            sha1 = ""
        else:
            data = adf.open(name, "r").read()
            sha1 = hashlib.sha1(data).hexdigest()
        if sha1:
            if len(name) > (80 - 4 - 2 - 1 - 2 - 2 - 40):
                print("    \"{0}\":\n    \"{1}\",".format(name, sha1))
            else:
                print("    \"{0}\": \"{1}\",".format(name, sha1))
        else:
            print("    \"{0}\": \"\",".format(name))
    print("")
    print(startup_sequence)


if __name__ == "__main__":
    main()



from fs_uae_launcher.ui.config.expand import expand_config, \
    AbstractExpandFunctions
from fs_uae_launcher.ui.config.model import ImplicitConfig, normalize, \
    create_model
from fs_uae_launcher.Config import Config
from fs_uae_launcher.Settings import Settings
from fsbc.util import unused
import fsui


class ConfigBrowser(fsui.VerticalItemView):

    def __init__(self, parent):
        fsui.VerticalItemView.__init__(self, parent)

        self.items = []
        self.game_icon = fsui.Image("fs_uae_launcher:res/16/controller.png")
        self.config_icon = fsui.Image(
            "fs_uae_launcher:res/fsuae_config_16.png")
        Config.add_listener(self)
        Settings.add_listener(self)
        # self.update_search()

        self.manual_download_icon = fsui.Image(
            "fs_uae_launcher:res/16/arrow_down_yellow.png")
        self.auto_download_icon = fsui.Image(
            "fs_uae_launcher:res/16/arrow_down_green.png")
        self.blank_icon = fsui.Image(
            "fs_uae_launcher:res/16/blank.png")
        self.missing_color = fsui.Color(0xa8, 0xa8, 0xa8)

        self.platform_icons = {}
        self._need_update = True
        self.update_items()

    def update_items(self):
        # FIXME: replace with a timer-scheduled function so this function does
        # not run needlessly often
        self._need_update = True
        fsui.call_after(self.do_update_items)

    def do_update_items(self):
        if not self._need_update:
            return
        print("do_update_items")
        implicit = ImplicitConfig(ConfigProxy(), SettingsProxy())
        expand_config(implicit, ExpandFunctions())
        model = create_model(implicit, show_all=False)
        # print_model(model)
        items = []

        def flatten(item_list, level):

            for model_item in item_list:
                if model_item.active:
                    text = "    " * level + str(model_item.text)
                    if model_item.extra:
                        text += " [" + model_item.extra + "]"
                    items.append(text)
                if model_item.children:
                    flatten(model_item.children, level + 1)

        flatten(model.items, 0)
        print(items)
        self.set_items(items)
        self._need_update = False

    def on_destroy(self):
        Config.remove_listener(self)
        Settings.remove_listener(self)

    def on_select_item(self, index):
        if index is None:
            return
        pass

    def on_activate_item(self, index):
        pass

    def on_config(self, key, value):
        unused(key)
        unused(value)
        self.update_items()

    def on_setting(self, key, value):
        unused(key)
        unused(value)
        self.update_items()

    def set_items(self, items):
        self.items = items
        self.update()

    def get_item_count(self):
        return len(self.items)

    def get_item_text(self, index):
        item = self.items[index]
        return item
        # return ""

    # def get_item_search_text(self, index):
    #     # return self.items[index][3]
    #     # FIXME: lower-case search string?
    #     return self.items[index][str("sort_key")]
    #
    # def get_item_text_color(self, index):
    #     have = self.items[index][str("have")]
    #     if not have:
    #         return self.missing_color

    def get_item_icon(self, index):
        # item = self.items[index]
        # platform_id = (item[str("platform")] or "").lower()
        # if item[str("have")] == 1:
        #     return self.manual_download_icon
        # elif item[str("have")] == 0:
        #     return self.blank_icon
        # elif item[str("have")] == 2:
        #     return self.auto_download_icon
        # elif item[str("have")] == 4:
        #     if platform_id not in self.platform_icons:
        #         try:
        #             icon = fsui.Image("fs_uae_launcher:res/16/{0}.png".format(
        #                 platform_id))
        #         except Exception:
        #             icon = self.game_icon
        #         self.platform_icons[platform_id] = icon
        #     return self.platform_icons[platform_id]
        # else:
        return self.config_icon

    # def on_get_item_tooltip(self, row, column):
    #     return ""


class SettingsProxy:

    def get(self, key):  # , default=""):
        return Settings.get(key)  # , default)

    def __getitem__(self, item):
        return self.get(item)

    def __setitem__(self, key, value):
        Settings.set(key, value)


class ConfigProxy:

    def get(self, key):
        return Config.get(key)

    def __getitem__(self, item):
        return self.get(item)

    def __setitem__(self, key, value):
        Config.set(key, value)


class ExpandFunctions(AbstractExpandFunctions):

    @staticmethod
    def matches(a, b):
        a = normalize(a)
        if isinstance(b, list):
            for b_item in b:
                if a == normalize(b_item):
                    return True
            return False
        return a == normalize(b)

    @staticmethod
    def fail(message):
        pass

    @staticmethod
    def warning(message):
        # warnings.append(message)
        pass

    @staticmethod
    def lower(s):
        return s.lower()



import locale
# we must set the LC_NUMERIC locale to C, because PyOpenGL has
# a problem if comma is used as a decimal separator (problem
# parsing the OpenGL version)
locale.setlocale(locale.LC_NUMERIC, str("C"))

from OpenGL.GL import *
# noinspection PyUnresolvedReferences
from OpenGL.GLU import *
# noinspection PyUnresolvedReferences
from OpenGL.GL.ARB.shader_objects import *
# noinspection PyUnresolvedReferences
from OpenGL.GL.ARB.vertex_shader import *
# noinspection PyUnresolvedReferences
from OpenGL.GL.ARB.multitexture import *
# noinspection PyUnresolvedReferences
from OpenGL.GL.ARB.fragment_shader import *
# noinspection PyUnresolvedReferences
from OpenGL.GL.ARB.framebuffer_object import *
# noinspection PyUnresolvedReferences
from OpenGL.raw.GL.EXT.framebuffer_object import *
# noinspection PyUnresolvedReferences
from OpenGL.arrays.vbo import VBO
# from OpenGLContext.arrays import *


# these constants are repeated here to aid code inspection tools (in
# pyopengl they are expanded (run-time) from a string).

GL_BLEND = 0xBE2
GL_TEXTURE_2D = 0xDE1
GL_COMPILE_STATUS = 0x8b81
GL_LINK_STATUS = 0x8b82
GL_FRAGMENT_SHADER = 0x8b30
GL_VERTEX_SHADER = 0x8b31
GL_ONE = 0x1
GL_ONE_MINUS_SRC_COLOR = 0x301
GL_SRC_ALPHA = 0x302
GL_ONE_MINUS_SRC_ALPHA = 0x303
GL_RGB = 0x1907
GL_RGBA = 0x1908
GL_BGRA = 0x80E1
GL_TEXTURE_MAG_FILTER = 0x2800
GL_TEXTURE_MIN_FILTER = 0x2801
GL_FRAMEBUFFER_EXT = 0x8D40
GL_COLOR_ATTACHMENT0_EXT = 0x8CE0
GL_RENDERBUFFER_EXT = 0x8D41
GL_DEPTH_ATTACHMENT_EXT = 0x8D00
GL_LINEAR = 0x2601
GL_FRAMEBUFFER_COMPLETE_EXT = 0x8CD5
GL_DEPTH_COMPONENT24 = 0x81A6
GL_QUADS = 0x7
GL_DEPTH_TEST = 0xB71
GL_TEXTURE_2D_ARB = 0x207A
GL_COLOR_BUFFER_BIT = 0x4000
GL_DEPTH_BUFFER_BIT = 0x100
GL_MODELVIEW = 0x1700
GL_PROJECTION = 0x1701
GL_LIGHT_MODEL_TWO_SIDE = 0xB52
GL_LIGHT_MODEL_LOCAL_VIEWER = 0xB51
GL_LIGHT_MODEL_AMBIENT = 0xB53
GL_SEPARATE_SPECULAR_COLOR = 0x81FA
GL_LIGHT_MODEL_COLOR_CONTROL = 0x81F8
GL_LIGHTING = 0xB50
GL_LIGHT0 = 0x4000
GL_LIGHT1 = 0x4001
GL_LIGHT2 = 0x4002
GL_LIGHT3 = 0x4003
GL_LIGHT4 = 0x4004
GL_LIGHT5 = 0x4005
GL_LIGHT6 = 0x4006
GL_LIGHT7 = 0x4007
GL_AMBIENT = 0x1200
GL_DIFFUSE = 0x1201
GL_SPECULAR = 0x1202
GL_SHININESS = 0x1601
GL_EMISSION = 0x1600
GL_POSITION = 0x1203
GL_FRONT = 0x404
GL_BACK = 0x405
GL_POLYGON_OFFSET_FILL = 0x8037


# FIXME: cache


def fs_emu_blending(enable):
    if enable:
        glEnable(GL_BLEND)
    else:
        glDisable(GL_BLEND)


def fs_emu_texturing(enable):
    if enable:
        glEnable(GL_TEXTURE_2D)
    else:
        glDisable(GL_TEXTURE_2D)


def filter_global(x):
    if x.lower().startswith("gl"):
        return True
    return False


__all__ = ["fs_emu_texturing", "fs_emu_blending"]


for __x in list(globals()):
    if filter_global(__x):
        __all__.append(__x)


class GL:
    pass


gl = GL


for __x in list(globals()):
    if filter_global(__x):
        setattr(gl, __x, globals()[__x])


if False:
    # this fixes ImportError: No module named win32 when used with py2exe
    # and other problems related to required modules missing from pyOpenGL
    # noinspection PyUnresolvedReferences
    import ctypes
    # noinspection PyUnresolvedReferences
    import logging
    # noinspection PyUnresolvedReferences
    import OpenGL.platform.win32
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays._buffers
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays._numeric
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays._strings
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.arraydatatype
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.arrayhelpers
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.buffers
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.ctypesarrays
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.ctypesparameters
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.ctypespointers
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.formathandler
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.lists
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.nones
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.numbers
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.numeric
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.numericnames
    # noinspection PyUnresolvedReferences
    # import OpenGL.arrays.numpymodule
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.strings
    # noinspection PyUnresolvedReferences
    import OpenGL.arrays.vbo



import io
import os
import time
import json
import base64
import platform
from uuid import uuid4
from functools import wraps
from urllib.parse import urlencode
from http.client import HTTPConnection
from urllib.error import HTTPError
from fsbc.Application import app
from fsbc.task import Task
from fsbc.util import memoize
from fsgs.FSGSDirectories import FSGSDirectories


class NonRetryableHTTPError(HTTPError):
    pass


class BadRequestError(NonRetryableHTTPError):
    pass


class UnauthorizedError(NonRetryableHTTPError):
    pass


class ForbiddenError(NonRetryableHTTPError):
    pass


class NotFoundError(NonRetryableHTTPError):
    pass


def retry(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        for i in range(10):
            if i > 0:
                print("retrying...")
            try:
                return f(*args, **kwargs)
            except NonRetryableHTTPError as e:
                raise e
            except Exception as e:
                print(repr(e))
                time.sleep(i * 0.5)
        return f(*args, **kwargs)
    return wrapper


class OGDClient(object):

    HTTPError = HTTPError
    BadRequestError = BadRequestError
    UnauthorizedError = UnauthorizedError
    ForbiddenError = ForbiddenError
    NotFoundError = NotFoundError
    NonRetryableHTTPError = NonRetryableHTTPError

    def __init__(self):
        pass

    @staticmethod
    def is_logged_in():
        # non-empty ogd_auth means we are logged in (probably, the
        # authorization can in theory have been invalidated on the server
        return bool(app.settings["database_auth"])

    def login_task(self, username, password):
        return LoginTask(self, username, password)

    def logout_task(self, auth_token):
        return LogoutTask(self, auth_token)

    @retry
    def auth(self, username, password, device_id, device_name):
        result = self.post(
            "/api/auth", {"username": username,
                          "password": password,
                          "device_id": device_id,
                          "device_name": device_name},
            auth=False)
        return result

    @retry
    def deauth(self, auth_token):
        result = self.post(
            "/api/deauth", {"auth_token": auth_token},
            auth=False)
        return result

    @staticmethod
    @memoize
    def get_server():
        server = app.settings["database_server"]
        if not server:
            p = os.path.join(FSGSDirectories.get_data_dir(), "Settings",
                             "database-server")
            if os.path.exists(p):
                with io.open(p, "r", encoding="UTF-8") as f:
                    server = f.read().strip()
        if not server:
            server = "oagd.net"
        return server

    @staticmethod
    def get_credentials():
        auth_token = app.settings["database_auth"]
        return "auth_token", auth_token

    def post(self, path, params=None, data=None, auth=True):
        headers = {
        }
        if auth:
            credentials = self.get_credentials()
            headers[str("Authorization")] = str("Basic " + base64.b64encode(
                "{0}:{1}".format(*credentials).encode("UTF-8")).decode("UTF-8"))

        # if data is None:
        #     data = "{}"
        server = self.get_server()
        connection = HTTPConnection(server, timeout=30)
        url = "http://{0}{1}".format(server, path)
        # if params:
        #     url += "?" + urlencode(params)
        if not data and params:
            data = urlencode(params)
            headers[str("Content-Type")] = \
                str("application/x-www-form-urlencoded")

        print(url, headers)
        if isinstance(data, dict):
            data = json.dumps(data)
        # print(data)
        connection.request(str("POST"), str(url), data, headers=headers)
        response = connection.getresponse()
        if response.status not in [200]:
            print(response.status, response.reason)
            if response.status == 400:
                class_ = BadRequestError
            elif response.status == 401:
                class_ = UnauthorizedError
            elif response.status == 403:
                class_ = ForbiddenError
            elif response.status == 404:
                class_ = NotFoundError
            else:
                class_ = HTTPError
            raise class_(url, response.status, response.reason,
                         response.getheaders(), None)
        data = response.read()
        if len(data) > 0 and data[0:1] == b"{":
            doc = json.loads(data.decode("UTF-8"))
            return doc
        return data


def get_device_name():
    try:
        return platform.node() or "Unknown Computer"
    except Exception:
        return "Unknown Computer"


class LoginTask(Task):

    def __init__(self, client, username, password):
        Task.__init__(self, "Login Task")
        self.client = client
        self.username = username
        self.password = password

    def run(self):
        self.progressed("Logging into oagd.net...")
        if not app.settings["device_id"]:
            app.settings["device_id"] = str(uuid4())
        try:
            result = self.client.auth(
                self.username, self.password, app.settings["device_id"],
                get_device_name())
        except UnauthorizedError:
            raise Task.Failure("Wrong e-mail address or password")

        app.settings["database_username"] = result["username"]
        app.settings["database_email"] = result["email"]
        app.settings["database_auth"] = result["auth_token"]
        app.settings["database_password"] = ""


class LogoutTask(Task):

    def __init__(self, client, auth_token):
        Task.__init__(self, "Logout Task")
        self.client = client
        self.auth_token = auth_token

    def run(self):
        self.progressed("Logging out from oagd.net...")
        if not app.settings["device_id"]:
            app.settings["device_id"] = str(uuid4())
        self.client.deauth(self.auth_token)

        app.settings["database_username"] = ""
        # app.settings["database_email"] = ""
        app.settings["database_auth"] = ""
        app.settings["database_password"] = ""



#!/usr/bin/env python
import os


lines = []
for dir_path, dir_names, file_names in os.walk("."):
    for name in file_names:
        if name in ["manifest.txt", "update-manifest.py"]:
            continue
        path = os.path.join(dir_path, name)
        lines.append(path[2:])
lines.sort()
with open ("manifest.txt", "wb") as f:
    for line in lines:
        f.write(line + "\n")



import sys
import platform
import fsbc.debug
import fsbc.logging
# import fsbc.unicode
from fsbc.Application import Application

init_called = False
unicode_patched = False
logging_enabled = False
exception_handler_enabled = False


def initialize_application(
        name=None, version=None, patch_unicode=True,
        enable_logging=True, enable_exception_handler=True):

    global init_called, unicode_patched, logging_enabled, \
        exception_handler_enabled
    init_called = True

    if name and enable_logging:
        fsbc.logging.setup_logging(name + ".log.txt")
        logging_enabled = True

    if name is not None:
        Application.app_name = name
        print(name)

    if version is not None:
        Application.app_version = version
        print(version)

    if enable_exception_handler:
        fsbc.debug.enable_exception_handler()
        exception_handler_enabled = True

    if patch_unicode:
        # patch system libraries (and argv) to work better with
        # unicode-enabled programs
        # FIXME: removed for Python 3
        # fsbc.unicode.patch_all()
        unicode_patched = True

    print(platform.uname())
    print(sys.argv)



from fsbc.util import unused
import fsui as fsui
from fsbc.Application import app
from fsgs.Database import Database
from ..Settings import Settings
from ..I18N import gettext


class GameListSelector(fsui.Choice):

    def __init__(self, parent):
        fsui.Choice.__init__(self, parent)
        self.game_lists = []
        self.populate_list()
        self.item_selected.connect(self.on_item_selected)

        Settings.add_listener(self)
        self.on_setting("game_list_uuid", app.settings["game_list_uuid"])

    def on_destroy(self):
        Settings.remove_listener(self)

    def on_setting(self, key, value):
        if key == "config_refresh":
            old_list_uuid = self.get_selected_list_uuid()
            print("- old list uuid", old_list_uuid)
            print("- set choice index to None")
            self.set_index(None)
            self.populate_list()
            print("- game lists", self.game_lists)
            for i, item in enumerate(self.game_lists):
                print("-", item[0])
                if item[0] == old_list_uuid:
                    if self.get_index() != i:
                        print("- set choice index to", i)
                        self.set_index(i, signal=True)
                    break
            else:
                # list uuid is no longer valid
                self.set_index(0)
        elif key == "game_list_uuid":
            for i, item in enumerate(self.game_lists):
                if item[0] == value:
                    if self.get_index() != i:
                        self.set_index(i)
                    break
            else:
                self.set_index(0)

    def populate_list(self):
        database = Database.instance()
        self.game_lists = database.get_game_lists()
        if len(self.game_lists) > 0:
            self.game_lists.insert(0, ["", self.ITEM_SEPARATOR])
        self.game_lists.insert(
            0, [Database.GAME_LIST_GAMES, gettext("All Games")])
        self.game_lists.insert(
            0, [Database.GAME_LIST_CONFIGS, gettext("All Configurations")])
        self.game_lists.insert(
            0, ["", gettext("All Games and Configurations")])

        self.clear()
        for item in self.game_lists:
            list_name = item[1]
            if list_name == "Favorites":
                list_name = gettext("Favorites")
            self.add_item(list_name)

    def get_selected_list_uuid(self):
        index = self.get_index()
        try:
            return self.game_lists[index][0]
        except IndexError:
            # should not really happen...
            return ""

    def on_item_selected(self, index):
        unused(index)
        list_uuid = self.get_selected_list_uuid()
        app.settings["game_list_uuid"] = list_uuid



import time
import threading
import traceback
from fsgs.Database import Database
from fsgs.ogd.context import SynchronizerContext
from fsgs.ogd.meta import MetaSynchronizer
import fsui as fsui
from .FileScanner import FileScanner
from .ConfigurationScanner import ConfigurationScanner
from .Config import Config
from .GameScanner import GameScanner
from .Settings import Settings
from .Signal import Signal

from .GameRatingSynchronizer import GameRatingSynchronizer
from fsgs.ogd.lists import ListsSynchronizer


class Scanner:

    running = False
    stop_flag = False
    status = ("", "")
    error = ""
    paths = []
    update_game_database = False
    purge_other_dirs = False
    scan_for_files = False

    @classmethod
    def stop_check(cls):
        return cls.stop_flag

    @classmethod
    def scan_thread(cls):
        try:
            with Database.get_instance() as database:
                cls._scan_thread(database)
        except Exception as e:
            cls.error = str(e)
            traceback.print_exc()
        # else:
        #     if cls.on_status:
        #         cls.on_status(("", _("Committing data...")))
        #     database.commit()

        def on_done():
            # FIXME: these should be removed soon
            Settings.set("last_scan", str(time.time()))
            Settings.set("config_refresh", str(time.time()))

            # this must be called from main, since callbacks are broadcast
            # when settings are changed

            Signal.broadcast("scan_done")
            Config.update_kickstart()
        # call on_done from main thread
        fsui.call_after(on_done)
        cls.running = False

    @classmethod
    def _scan_thread(cls, database):
        if cls.scan_for_files:
            scanner = FileScanner(
                cls.paths, cls.purge_other_dirs, on_status=cls.on_status,
                stop_check=cls.stop_check)
            scanner.scan()
            if cls.stop_check():
                return

        # if cls.scan_for_configs or
        # if cls.update_game_database:
        scanner = ConfigurationScanner(
            cls.paths, on_status=cls.on_status, stop_check=cls.stop_check)
        scanner.scan(database)

        if cls.update_game_database:

            context = SynchronizerContext()

            synchronizer = MetaSynchronizer(
                context, on_status=cls.on_status, stop_check=cls.stop_check)
            synchronizer.synchronize()

            synchronizer = GameRatingSynchronizer(
                context, database, on_status=cls.on_status,
                stop_check=cls.stop_check)
            synchronizer.username = "auth_token"
            synchronizer.password = Settings.get("database_auth")
            synchronizer.synchronize()

            synchronizer = ListsSynchronizer(
                context, on_status=cls.on_status, stop_check=cls.stop_check)
            synchronizer.synchronize()

            scanner = GameScanner(
                context, cls.paths, on_status=cls.on_status,
                stop_check=cls.stop_check)
            scanner.update_game_database()

        scanner = GameScanner(
            None, cls.paths, on_status=cls.on_status,
            stop_check=cls.stop_check)
        scanner.scan(database)

    @classmethod
    def start(cls, paths, scan_for_files=True, update_game_database=False,
              purge_other_dirs=False):
        print("Scanner.start")
        if cls.running:
            print("scan already in progress")
            return
        cls.paths = paths[:]
        cls.running = True
        cls.error = ""
        cls.stop_flag = False
        cls.status = ("", "")
        # cls.scan_for_roms = scan_for_roms
        cls.scan_for_files = scan_for_files
        cls.purge_other_dirs = purge_other_dirs
        # cls.scan_for_configs = scan_for_configs
        cls.update_game_database = update_game_database
        threading.Thread(target=cls.scan_thread, name="ScannerThread").start()

    @classmethod
    def on_status(cls, status):
        cls.status = status



#!/usr/bin/env python3

import os
import sys
import subprocess

p = subprocess.Popen("file --dereference `which python3`",
                     shell=True, stdout=subprocess.PIPE)
exe_info = p.stdout.read().decode("UTF-8")
if "386" in exe_info:
    # arch = "i386"
    arch = "x86"
elif "x86-64" in exe_info:
    # arch = "amd64"
    arch = "x86-64"
else:
    raise Exception("unrecognized arch")

# if os.environ.get("STEAM_RUNTIME", ""):
if os.environ.get("STEAMOS", ""):
    os_name = "steamos"
    # if arch == "i386":
    #     # steam runtime sdk compiles with -mtune=generic -march=i686
    #     arch = "i686"
else:
    os_name = "linux"

package_name = "fs-uae-launcher"
package_version = sys.argv[1]
dbg_package_dir = "{0}-dbg_{1}_{2}_{3}".format(
    package_name, package_version, os_name, arch)
package_dir = "../{2}/{0}_{1}_{2}_{3}".format(
    package_name, package_version, os_name, arch)
full_package_name = "{0}_{1}_{2}_{3}".format(
    package_name, package_version, os_name, arch)


def s(command):
    c = command.format(**globals())
    print(c)
    assert os.system(c) == 0


def wrap(name, target, args=None):
    if args is None:
        args = ["$@"]
    path = os.path.join(package_dir, name)
    with open(path, "w") as f:
        f.write("#!/bin/sh\n")
        f.write("MYDIR=$(dirname \"$0\")\n")
        f.write("export LD_LIBRARY_PATH=\"$MYDIR:$LD_LIBRARY_PATH\"\n")
        command = "\"$MYDIR/{0}\"".format(target)
        for arg in args:
            command += " \"{0}\"".format(arg)
        if os_name == "steamos":
            # if arch == "i686":
            if arch == "x86":
                bin_dir = "bin32"
            # elif arch == "amd64":
            elif arch == "x86-64":
                bin_dir = "bin64"
            else:
                raise Exception("unsupported steamos arch?")
            f.write("if [ -e \"$HOME/.steam/{0}/steam-runtime/"
                    "run.sh\" ]; then\n".format(bin_dir))
            f.write("RUN_SH=\"$HOME/.steam/{0}/steam-runtime/"
                    "run.sh\"\n".format(bin_dir))
            f.write("else\n")
            f.write("RUN_SH=\"/home/steam/.steam/{0}/steam-runtime/"
                    "run.sh\"\n".format(bin_dir))
            f.write("fi\n")
            f.write("exec $RUN_SH {0}\n".format(command))
        else:
            f.write("exec {0}\n".format(command))
    os.chmod(path, 0o755)


s("rm -Rf {package_dir}")
# s("make -C ..")
# s("rm -Rf ../build ../dist")
s("cd ../.. && python3 setup.py build_exe")
s("mv ../../build/exe.linux-*-3.4 {package_dir}")

# we want to perform our own standalone/library management, so we remove the
# libraries added by cx_Freeze
s("rm -f {package_dir}/*.so.*")
s("rm -f {package_dir}/imageformats/*.so.*")

s("./standalone.py {package_dir}/platforms")
s("mv {package_dir}/platforms/*.so.* {package_dir}")
s("./standalone.py {package_dir}/imageformats")
s("mv {package_dir}/imageformats/*.so.* {package_dir}")
s("./standalone.py {package_dir}")
s("strip {package_dir}/*.so.*")
s("strip {package_dir}/*.so")
s("strip {package_dir}/imageformats/*.so")
s("chmod a-x {package_dir}/*.so")
s("cd ../.. && make")
s("cp -a ../../share {package_dir}")

s("mv {package_dir}/fs-uae-launcher {package_dir}/fs-uae-launcher.bin")
wrap("fs-uae-launcher", "fs-uae-launcher.bin")
wrap("fs-uae-arcade", "fs-uae-launcher.bin", ["--fs-uae-arcade", "$@"])

s("cd {package_dir} && tar Jcfv ../../../{full_package_name}.tar.xz *")

# s("rm -Rf {dbg_package_dir}")
# s("mkdir {dbg_package_dir}")
# s("cp -a ../fs-uae.dbg {dbg_package_dir}/")
# s("cp -a ../fs-uae-device-helper.dbg {dbg_package_dir}/")
# s("cd {dbg_package_dir} && tar Jcfv ../../{dbg_package_dir}.tar.xz *")



from fsgs.platform import PlatformHandler
from fsgs.mednafen.turbografx_16 import TurboGrafx16Runner
from .loader import SimpleLoader


class TurboGrafx16PlatformHandler(PlatformHandler):

    PLATFORM_NAME = "TurboGrafx-16"

    def __init__(self):
        PlatformHandler.__init__(self)

    def get_loader(self, fsgs):
        return TurboGrafx16Loader(fsgs)

    def get_runner(self, fsgs):
        return TurboGrafx16Runner(fsgs)


class TurboGrafx16Loader(SimpleLoader):
    pass



from PIL import Image


def fix(name, w, h):
    with open(name + ".raw", "rb") as f:
        data = f.read()
    im = Image.fromstring("RGBA", (w, h), data)
    im.save(name + ".png")


fix("title_font", 1024, 132)
fix("menu_font", 1024, 132)



import os
from PIL import Image


for item in os.listdir('.'):
    if not item.endswith('.png'):
        continue
    im = Image.open(item)
    if im.mode != "RGB":
        im = im.convert("RGB")
        im.save(item)



from fsgs.mednafen.mednafen import MednafenRunner


class GameBoyColorRunner(MednafenRunner):

    CONTROLLER = {
        "type": "gamepad",
        "description": "Built-in Controller",
        "mapping_name": "gameboycolor",
    }

    PORTS = [
        {
            "description": "Controller",
            "types": [CONTROLLER]
        },
    ]

    def __init__(self, fsgs):
        super().__init__(fsgs)

    def get_game_refresh_rate(self):
        return 59.73

    def mednafen_input_mapping(self, _):
        return {
            "A": "gb.input.builtin.gamepad.a",
            "B": "gb.input.builtin.gamepad.b",
            "UP": "gb.input.builtin.gamepad.up",
            "DOWN": "gb.input.builtin.gamepad.down",
            "LEFT": "gb.input.builtin.gamepad.left",
            "RIGHT": "gb.input.builtin.gamepad.right",
            "SELECT": "gb.input.builtin.gamepad.select",
            "START": "gb.input.builtin.gamepad.start",
        }

    def mednafen_rom_extensions(self):
        return [".gbc"]

    def mednafen_scanlines_setting(self):
        return 33

    def mednafen_special_filter(self):
        return "nn2x"

    def mednafen_system_prefix(self):
        return "gb"

    # def mednafen_video_size(self):
    #     return 160, 144



import fsui.qt
from .Widget import Widget


class ImageButton(fsui.qt.QPushButton, Widget):

    activated = fsui.qt.QSignal()

    def __init__(self, parent, image):
        fsui.qt.QPushButton.__init__(self, parent.get_container())
        self.init_widget(parent)
        self.setIcon(image.qicon)
        self.clicked.connect(self.__clicked)

    def set_image(self, image):
        self.setIcon(image.qicon)

    def __clicked(self):
        self.on_activate()
        self.activated.emit()

    def on_activate(self):
        pass



import fsui as fsui
from fsui.extra.iconheader import NewIconHeader
from ...I18N import gettext
from .PreferredJoysticksGroup import PreferredJoysticksGroup
from fs_uae_workspace.shell import shell_open
from fs_uae_launcher.DeviceManager import DeviceManager


class JoystickSettingsPage(fsui.Panel):

    def __init__(self, parent):
        fsui.Panel.__init__(self, parent)
        self.layout = fsui.VerticalLayout()
        # self.layout.set_padding(20, 20, 20, 20)

        self.icon_header = NewIconHeader(
            self, fsui.Icon("joystick-settings", "pkg:fs_uae_workspace"),
            gettext("Joystick Settings"),
            gettext("Configure joysticks and set preferred joystick devices"))
        self.layout.add(self.icon_header, fill=True, margin_bottom=20)

        self.list_view = fsui.ListView(self)
        self.list_view.set_min_height(140)
        self.list_view.item_activated.connect(self.on_joystick_activated)
        image = fsui.Image("fs_uae_workspace:res/16/gamepad.png")
        for device_name in DeviceManager.get_joystick_names():
            if DeviceManager.is_joystick(device_name):
                self.list_view.add_item(device_name, icon=image)
        self.layout.add(self.list_view, fill=True, expand=True)

        label = fsui.Label(
            self, gettext("Double-click a device entry to configure it (map "
                          "joystick buttons)."))
        self.layout.add(label, margin_top=10)

        self.layout.add_spacer(20)
        self.pref_group = PreferredJoysticksGroup(self)
        self.layout.add(self.pref_group, fill=True)

    def on_joystick_activated(self, index):
        device_name = self.list_view.get_item(index)
        print(self.get_window())
        shell_open("Workspace:Tools/JoystickConfig", [device_name],
                   parent=self.get_window())



import weakref
import fsui as fsui
from ...I18N import gettext
from ...Settings import Settings
from ...Signal import Signal
from ..PagedDialog import PagedDialog
from .advanced import AdvancedSettingsPage
from .advanced_video import AdvancedVideoSettingsPage
from .AudioSettingsPage import AudioSettingsPage
# from .CustomSettingsPage import CustomSettingsPage
from .ExperimentalFeaturesPage import ExperimentalFeaturesPage
# from .FilterSettingsPage import FilterSettingsPage
from .GameDatabaseSettingsPage import GameDatabaseSettingsPage
from .joystick import JoystickSettingsPage
from .keyboard import KeyboardSettingsPage
from .language import LanguageSettingsPage
from .logging import LoggingSettingsPage
from .maintenance import MaintenanceSettingsPage
from .mouse import MouseSettingsPage
from .NetplaySettingsPage import NetplaySettingsPage
from .ScanSettingsPage import ScanSettingsPage
from .video_sync import VideoSyncSettingsPage
from .VideoSettingsPage import VideoSettingsPage
from .whdload import WHDLoadSettingsPage
from fsui.qt import Qt


SPACE = ""


class SettingsDialog(PagedDialog):

    weak_instance = None  # type: SettingsDialog

    @classmethod
    def open(cls, parent):
        if cls.weak_instance is not None:
            # noinspection PyCallingNonCallable
            instance = cls.weak_instance()
            if instance is not None:
                instance.raise_and_activate()
                return
        instance = SettingsDialog(parent)
        instance.show()
        cls.weak_instance = weakref.ref(instance)

    def __del__(self):
        print("SettingsDialog.__del__")

    def __init__(self, parent, index=0):
        PagedDialog.__init__(self, parent, gettext("FS-UAE Launcher Settings"))

        # FIXME: remove this once the dialog uses Window as base class
        self.setAttribute(Qt.WA_DeleteOnClose, True)

        self.add_page(
            gettext("Appearance"), LanguageSettingsPage,
            # gettext("Language"), LanguageSettingsPage,
            fsui.Icon("language-settings", "pkg:fs_uae_workspace"))
        self.add_page(
            gettext("Joystick"), JoystickSettingsPage,
            fsui.Icon("joystick-settings", "pkg:fs_uae_workspace"))
        self.add_page(
            gettext("Keyboard"), KeyboardSettingsPage,
            fsui.Icon("keyboard-settings", "pkg:fs_uae_workspace"))
        self.add_page(
            gettext("Mouse"), MouseSettingsPage,
            fsui.Icon("mouse-settings", "pkg:fs_uae_workspace"))
        self.add_page(
            gettext("Audio"), AudioSettingsPage,
            fsui.Icon("audio-settings", "pkg:fs_uae_workspace"))
        self.add_page(
            gettext("Video"), VideoSettingsPage,
            fsui.Icon("video-settings", "pkg:fs_uae_workspace"))
        self.add_page(
            SPACE + gettext("Synchronization"), VideoSyncSettingsPage,
            fsui.Icon("blank", "pkg:fs_uae_launcher"))
        self.add_page(
            SPACE + gettext("Advanced"), AdvancedVideoSettingsPage,
            fsui.Icon("blank", "pkg:fs_uae_launcher"))
        # self.add_page(
        #     gettext("Filters & Scaling"), FilterSettingsPage,
        #     fsui.Icon("video-settings", "pkg:fs_uae_workspace"))
        # self.add_page(gettext("OpenGL Settings"), OpenGLSettingsPage)
        # if Settings.get("database_feature") == "1":
        self.add_page(
            gettext("File Database"), ScanSettingsPage,
            fsui.Icon("indexing-settings", "pkg:fs_uae_workspace"))
        self.add_page(
            gettext("Game Database"), GameDatabaseSettingsPage,
            fsui.Icon("database-settings", "pkg:fs_uae_workspace"))
        # self.add_page(gettext("Custom Settings"), CustomSettingsPage)
        self.add_page(
            gettext("Maintenance"), MaintenanceSettingsPage,
            fsui.Icon("maintenance", "pkg:fs_uae_workspace"))
        if Settings.get("netplay_feature") == "1":
            self.add_page(
                gettext("Net Play"), NetplaySettingsPage,
                fsui.Icon("netplay-settings", "pkg:fs_uae_workspace"))
        self.add_page(
            "WHDLoad", WHDLoadSettingsPage,
            fsui.Icon("settings", "pkg:fs_uae_workspace"))
        self.add_page(
            gettext("Logging"), LoggingSettingsPage,
            fsui.Icon("settings", "pkg:fs_uae_workspace"))
        self.add_page(
            gettext("Experimental Features"), ExperimentalFeaturesPage,
            fsui.Icon("settings", "pkg:fs_uae_workspace"))
        self.add_page(
            gettext("Advanced Settings"), AdvancedSettingsPage,
            fsui.Icon("settings", "pkg:fs_uae_workspace"))

        last_index = self.get_page_index_by_title(
            Settings.get("last_settings_page"))
        index = last_index or index
        self.list_view.set_index(index)

        self.set_size((900, 560))
        # self.center_on_parent()

        self.closed.connect(self.__closed)
        self.page_changed.connect(self.__page_changed)

    def __page_changed(self):
        index = self.get_index()
        Settings.set("last_settings_page", self.get_page_title(index))

    def __closed(self):
        Signal.broadcast("settings_updated")

    # def on_close(self):
    #     self.destroy()



import json
from fsgs.mame.mame import MAMERunner


class ArcadeRunner(MAMERunner):

    CONTROLLER = {
        "type": "controller",
        "description": "Controller",
        "mapping_name": "arcade",
    }

    PORTS = [
        {
            "description": "1st Controller",
            "types": [CONTROLLER]
        }, {
            "description": "2nd Controller",
            "types": [CONTROLLER]
        }, {
            "description": "3rd Controller",
            "types": [CONTROLLER]
        }, {
            "description": "4th Controller",
            "types": [CONTROLLER]
        },
    ]

    def mame_romset(self):
        name = self.config["mame_rom_set"]
        files = {}
        for entry in json.loads(self.config["file_list"]):
            files[entry["sha1"]] = entry["name"]
        return name, files

    def mame_input_mapping(self, _):
        mapping = {
            "START": "START#",
            "SELECT": "COIN#",
            "UP": "P#_JOYSTICK_UP",
            "DOWN": "P#_JOYSTICK_DOWN",
            "LEFT": "P#_JOYSTICK_LEFT",
            "RIGHT": "P#_JOYSTICK_RIGHT",
        }
        button_order = self.config.get(
            "button_order", "1 2 3 4 5 6 7 8 9 10 11 12")
        b = 0
        for button in button_order.split(" "):
            button = button.strip()
            if button:
                b += 1
                mapping[str(b)] = "P#_BUTTON" + button
        return mapping



import os
import tempfile
import traceback
import subprocess
from fsbc.system import windows, macosx
from fsbc.Application import Application, app

try:
    getcwd = os.getcwdu
except AttributeError:
    getcwd = os.getcwd


class FSUAE(object):

    @classmethod
    def start_with_config(cls, config):
        print("FSUAE.start_with_config:")
        tf = tempfile.NamedTemporaryFile(suffix=".fs-uae", delete=False)
        config_file = tf.name
        with tf:
            for line in config:
                print(line)
                tf.write(line.encode("UTF-8"))
                tf.write(b"\n")
        args = [config_file]
        return cls.start_with_args(args), config_file

    @classmethod
    def start_with_args(cls, args, **kwargs):
        print("FSUAE.start_with_args:", args)
        exe = cls.find_executable()
        print("current dir (cwd): ", getcwd())
        print("using fs-uae executable:", exe)
        args = [exe] + args

        # if "--always-on-top" in sys.argv:
        #     args += ["--always-on-top"]

        print(args)
        for name in ["SDL_VIDEODRIVER", "__GL_SYNC_TO_VBLANK"]:
            if name in os.environ:
                print("{0} was present in environment, removing!".format(name))
                del os.environ[name]
        env = os.environ.copy()
        print(repr(env))
        # env = None
        try:
            cls.center_window(args, env)
        except Exception:
            traceback.print_exc()
        cls.add_environment_from_settings(env)
        # env[str("SDL_VIDEO_WINDOW_POS")] = "0,0"
        # args += ["--fullscreen-mode", "desktop"]
        if windows:
            p = subprocess.Popen(args, env=env, close_fds=True, **kwargs)
        else:
            p = subprocess.Popen(args, env=env, **kwargs)
        return p

    @classmethod
    def add_environment_from_settings(cls, env):
        for key, value in app.settings.values.items():
            # if not key.startswith("environment_"):
            #     continue
            # key = key[12:].upper()
            if not key.isupper():
                continue
            # Check if it looks like a valid environment variable
            for c in key:
                if c not in "ABCDEFGHIJKLMNOPQRSTUVWXYZ_0123456789":
                    break
            else:
                print("[ENV] {} = {}".format(key, value))
                env[key] = value

    @classmethod
    def center_window(cls, args, env):
        # FIXME: does not really belong here (dependency loop)
        from fs_uae_launcher.Config import Config
        from fs_uae_launcher.Settings import Settings
        width = Config.get("window_width") or Settings.get("window_width")
        height = Config.get("window_height") or Settings.get("window_height")
        try:
            width = int(width)
        except:
            width = 960
        try:
            height = int(height)
        except:
            height = 540
        from fs_uae_launcher.ui.MainWindow import MainWindow
        if MainWindow.instance is None:
            return

        main_w, main_h = MainWindow.instance.get_size()
        main_x, main_y = MainWindow.instance.get_position()

        x = main_x + (main_w - width) // 2
        y = main_y + (main_h - height) // 2

        # FIXME: reimplement without wx
        # if windows:
        #     import wx
        #    y += wx.SystemSettings_GetMetric(wx.SYS_CAPTION_Y)

        env[str("SDL_VIDEO_WINDOW_POS")] = str("{0},{1}".format(x, y))
        args.append("--window-x={0}".format(x))
        args.append("--window-y={0}".format(y))
        # print("window position", env["SDL_VIDEO_WINDOW_POS"])
        # os.environ["SDL_VIDEO_WINDOW_POS"] = "{0},{1}".format(x, y)

    @classmethod
    def find_executable(cls, executable="fs-uae"):
        application = Application.instance()

        if os.path.isdir("../fs-uae/src"):
            # running FS-UAE Launcher from source directory, we
            # then want to run the locally compiled fs-uae binary
            path = "../fs-uae/" + executable
            if windows:
                path += ".exe"
            if os.path.isfile(path):
                return path
            raise Exception("Could not find development FS-UAE executable")

        if windows:
            exe = os.path.join(
                application.executable_dir(), executable + ".exe")
            if not os.path.exists(exe):
                exe = os.path.join(
                    application.executable_dir(),
                    "fs-uae/" + executable + ".exe")
            if not os.path.exists(exe):
                exe = os.path.join(
                    application.executable_dir(), "../" + executable + ".exe")
        elif macosx:
            exe = os.path.join(application.executable_dir(), executable)
            if not os.path.exists(exe):
                exe = os.path.join(
                    application.executable_dir(),
                    "../FS-UAE.app/Contents/MacOS/" + executable)
            if not os.path.exists(exe):
                exe = os.path.join(
                    application.executable_dir(),
                    "../../../FS-UAE.app/Contents/MacOS/" + executable)
            if not os.path.exists(exe):
                exe = os.path.join(
                    application.executable_dir(),
                    "../../../Programs/Mac OS X/FS-UAE.app/Contents/MacOS/" + executable)
            if not os.path.exists(exe):
                exe = os.path.join(
                    application.executable_dir(),
                    "../../../FS-UAE Launcher.app/Contents/Resources/"
                    "FS-UAE.app/Contents/MacOS/" + executable)
        else:
            print("application executable dir", application.executable_dir())
            exe = os.path.join(application.executable_dir(), executable)
            print("checking", exe)
            if not os.path.exists(exe):
                exe = os.path.join(
                    application.executable_dir(), "..", "bin", executable)
                print("checking", exe)
            if not os.path.exists(exe):
                return executable

        if not os.path.exists(exe):
            raise Exception("Could not find {0} executable".format(
                repr(executable)))
        return exe
